@@ -30,6 +30,8 @@ public:
 
   static BluetoothHfpManager* Get();
   virtual ~BluetoothHfpManager() { }
+  static void InitHfpInterface(BluetoothProfileResultHandler* aRes);
+  static void DeinitHfpInterface(BluetoothProfileResultHandler* aRes);
 
   bool ConnectSco();
   bool DisconnectSco();
