@@ -22,4 +22,4 @@ struct ParamTraits<mozilla::dom::bluetooth::BluetoothObjectType>
 
 } // namespace IPC
 
-#endif // mozilla_dom_bluetooth_ipc_bluetoothchild_h__
+#endif // mozilla_dom_bluetooth_ipc_bluetoothmessageutils_h__
