@@ -25,6 +25,7 @@
 #include "mozilla/dom/XMLDocument.h"
 #include "nsIStreamListener.h"
 #include "ChildIterator.h"
+#include "nsITimer.h"
 
 #include "nsXBLBinding.h"
 #include "nsXBLPrototypeBinding.h"
@@ -45,10 +46,10 @@
 #include "nsIScriptContext.h"
 #include "xpcpublic.h"
 #include "jswrapper.h"
-#include "nsCxPusher.h"
 
 #include "nsThreadUtils.h"
 #include "mozilla/dom/NodeListBinding.h"
+#include "mozilla/dom/ScriptSettings.h"
 
 using namespace mozilla;
 using namespace mozilla::dom;
@@ -216,17 +217,22 @@ nsBindingManager::RemovedFromDocumentInternal(nsIContent* aContent,
 {
   NS_PRECONDITION(aOldDocument != nullptr, "no old document");
 
-  if (mDestroyed)
-    return;
-
   nsRefPtr<nsXBLBinding> binding = aContent->GetXBLBinding();
   if (binding) {
-    binding->PrototypeBinding()->BindingDetached(binding->GetBoundElement());
-    binding->ChangeDocument(aOldDocument, nullptr);
+    // The binding manager may have been destroyed before a runnable
+    // has had a chance to reach this point. If so, we bail out on calling
+    // BindingDetached (which may invoke a XBL destructor) and
+    // ChangeDocument, but we still want to clear out the binding
+    // and insertion parent that may hold references.
+    if (!mDestroyed) {
+      binding->PrototypeBinding()->BindingDetached(binding->GetBoundElement());
+      binding->ChangeDocument(aOldDocument, nullptr);
+    }
+
     aContent->SetXBLBinding(nullptr, this);
   }
 
-  // Clear out insertion parents and content lists.
+  // Clear out insertion parent and content lists.
   aContent->SetXBLInsertionParent(nullptr);
 }
 
@@ -329,7 +335,7 @@ nsBindingManager::RemoveFromAttachedQueue(nsXBLBinding* aBinding)
 {
   // Don't remove items here as that could mess up an executing
   // ProcessAttachedQueue. Instead, null the entry in the queue.
-  uint32_t index = mAttachedStack.IndexOf(aBinding);
+  size_t index = mAttachedStack.IndexOf(aBinding);
   if (index != mAttachedStack.NoIndex) {
     mAttachedStack[index] = nullptr;
   }
@@ -338,8 +344,7 @@ nsBindingManager::RemoveFromAttachedQueue(nsXBLBinding* aBinding)
 nsresult
 nsBindingManager::AddToAttachedQueue(nsXBLBinding* aBinding)
 {
-  if (!mAttachedStack.AppendElement(aBinding))
-    return NS_ERROR_OUT_OF_MEMORY;
+  mAttachedStack.AppendElement(aBinding);
 
   // If we're in the middle of processing our queue already, don't
   // bother posting the event.
@@ -365,6 +370,16 @@ nsBindingManager::PostProcessAttachedQueueEvent()
   }
 }
 
+// static
+void
+nsBindingManager::PostPAQEventCallback(nsITimer* aTimer, void* aClosure)
+{
+  nsRefPtr<nsBindingManager> mgr = 
+    already_AddRefed<nsBindingManager>(static_cast<nsBindingManager*>(aClosure));
+  mgr->PostProcessAttachedQueueEvent();
+  NS_RELEASE(aTimer);
+}
+
 void
 nsBindingManager::DoProcessAttachedQueue()
 {
@@ -379,7 +394,19 @@ nsBindingManager::DoProcessAttachedQueue()
     // Someone's doing event processing from inside a constructor.
     // They're evil, but we'll fight back!  Just poll on them being
     // done and repost the attached queue event.
-    PostProcessAttachedQueueEvent();
+    //
+    // But don't poll in a tight loop -- otherwise we keep the Gecko
+    // event loop non-empty and trigger bug 1021240 on OS X.
+    nsresult rv = NS_ERROR_FAILURE;
+    nsCOMPtr<nsITimer> timer = do_CreateInstance(NS_TIMER_CONTRACTID);
+    if (timer) {
+      rv = timer->InitWithFuncCallback(PostPAQEventCallback, this,
+                                       100, nsITimer::TYPE_ONE_SHOT);
+    }
+    if (NS_SUCCEEDED(rv)) {
+      NS_ADDREF_THIS();
+      NS_ADDREF(timer);
+    }
   }
 
   // No matter what, unblock onload for the event that's fired.
@@ -462,7 +489,7 @@ nsBindingManager::PutXBLDocumentInfo(nsXBLDocumentInfo* aDocumentInfo)
   NS_PRECONDITION(aDocumentInfo, "Must have a non-null documentinfo!");
 
   if (!mDocumentTable) {
-    mDocumentTable = new nsRefPtrHashtable<nsURIHashKey,nsXBLDocumentInfo>(16);
+    mDocumentTable = new nsRefPtrHashtable<nsURIHashKey,nsXBLDocumentInfo>();
   }
 
   mDocumentTable->Put(aDocumentInfo->DocumentURI(), aDocumentInfo);
@@ -493,7 +520,8 @@ nsBindingManager::PutLoadingDocListener(nsIURI* aURL, nsIStreamListener* aListen
   NS_PRECONDITION(aListener, "Must have a non-null listener!");
 
   if (!mLoadingDocTable) {
-    mLoadingDocTable = new nsInterfaceHashtable<nsURIHashKey,nsIStreamListener>(16);
+    mLoadingDocTable =
+      new nsInterfaceHashtable<nsURIHashKey,nsIStreamListener>();
   }
   mLoadingDocTable->Put(aURL, aListener);
 
@@ -610,21 +638,9 @@ nsBindingManager::GetBindingImplementation(nsIContent* aContent, REFNSIID aIID,
 
       // We have never made a wrapper for this implementation.
       // Create an XPC wrapper for the script object and hand it back.
-
-      nsIDocument* doc = aContent->OwnerDoc();
-
-      nsCOMPtr<nsIScriptGlobalObject> global =
-        do_QueryInterface(doc->GetWindow());
-      if (!global)
-        return NS_NOINTERFACE;
-
-      nsIScriptContext *context = global->GetContext();
-      if (!context)
-        return NS_NOINTERFACE;
-
-      AutoPushJSContext cx(context->GetNativeContext());
-      if (!cx)
-        return NS_NOINTERFACE;
+      AutoJSAPI jsapi;
+      jsapi.Init();
+      JSContext* cx = jsapi.cx();
 
       nsIXPConnect *xpConnect = nsContentUtils::XPConnect();
 
@@ -634,11 +650,11 @@ nsBindingManager::GetBindingImplementation(nsIContent* aContent, REFNSIID aIID,
       // If we're using an XBL scope, we need to use the Xray view to the bound
       // content in order to view the full array of methods defined in the
       // binding, some of which may not be exposed on the prototype of
-      // untrusted content.
+      // untrusted content. We don't need to consider add-on scopes here
+      // because they're chrome-only and no Xrays are involved.
       //
       // If there's no separate XBL scope, or if the reflector itself lives in
-      // the XBL scope, we'll end up with the global of the reflector, and this
-      // will all be a no-op.
+      // the XBL scope, we'll end up with the global of the reflector.
       JS::Rooted<JSObject*> xblScope(cx, xpc::GetXBLScopeOrGlobal(cx, jsobj));
       NS_ENSURE_TRUE(xblScope, NS_ERROR_UNEXPECTED);
       JSAutoCompartment ac(cx, xblScope);
@@ -809,23 +825,17 @@ static PLDHashOperator
 EnumAppendAllSheets(nsRefPtrHashKey<nsIContent> *aKey, void* aClosure)
 {
   nsIContent *boundContent = aKey->GetKey();
-  nsTArray<nsCSSStyleSheet*>* array =
-    static_cast<nsTArray<nsCSSStyleSheet*>*>(aClosure);
+  nsTArray<CSSStyleSheet*>* array =
+    static_cast<nsTArray<CSSStyleSheet*>*>(aClosure);
   for (nsXBLBinding *binding = boundContent->GetXBLBinding(); binding;
        binding = binding->GetBaseBinding()) {
-    nsXBLPrototypeResources::sheet_array_type* sheets =
-      binding->PrototypeBinding()->GetStyleSheets();
-    if (sheets) {
-      // Copy from nsTArray<nsRefPtr<nsCSSStyleSheet> > to
-      // nsTArray<nsCSSStyleSheet*>.
-      array->AppendElements(*sheets);
-    }
+    binding->PrototypeBinding()->AppendStyleSheetsTo(*array);
   }
   return PL_DHASH_NEXT;
 }
 
 void
-nsBindingManager::AppendAllSheets(nsTArray<nsCSSStyleSheet*>& aArray)
+nsBindingManager::AppendAllSheets(nsTArray<CSSStyleSheet*>& aArray)
 {
   if (mBoundContentSet) {
     mBoundContentSet->EnumerateEntries(EnumAppendAllSheets, &aArray);
@@ -836,12 +846,12 @@ static void
 InsertAppendedContent(XBLChildrenElement* aPoint,
                       nsIContent* aFirstNewContent)
 {
-  uint32_t insertionIndex;
+  int32_t insertionIndex;
   if (nsIContent* prevSibling = aFirstNewContent->GetPreviousSibling()) {
     // If we have a previous sibling, then it must already be in aPoint. Find
     // it and insert after it.
     insertionIndex = aPoint->IndexOfInsertedChild(prevSibling);
-    MOZ_ASSERT(insertionIndex != aPoint->NoIndex);
+    MOZ_ASSERT(insertionIndex != -1);
 
     // Our insertion index is one after our previous sibling's index.
     ++insertionIndex;
@@ -849,7 +859,7 @@ InsertAppendedContent(XBLChildrenElement* aPoint,
     // Otherwise, we append.
     // TODO This is wrong for nested insertion points. In that case, we need to
     // keep track of the right index to insert into.
-    insertionIndex = aPoint->mInsertedChildren.Length();
+    insertionIndex = aPoint->InsertedChildrenLength();
   }
 
   // Do the inserting.
@@ -873,6 +883,19 @@ nsBindingManager::ContentAppended(nsIDocument* aDocument,
   // Try to find insertion points for all the new kids.
   XBLChildrenElement* point = nullptr;
   nsIContent* parent = aContainer;
+
+  // Handle appending of default content.
+  if (parent && parent->IsActiveChildrenElement()) {
+    XBLChildrenElement* childrenEl = static_cast<XBLChildrenElement*>(parent);
+    if (childrenEl->HasInsertedChildren()) {
+      // Appending default content that isn't being used. Ignore.
+      return;
+    }
+
+    childrenEl->MaybeSetupDefaultContent();
+    parent = childrenEl->GetParent();
+  }
+
   bool first = true;
   do {
     nsXBLBinding* binding = GetBindingWithContent(parent);
@@ -946,6 +969,18 @@ nsBindingManager::ContentRemoved(nsIDocument* aDocument,
 
   XBLChildrenElement* point = nullptr;
   nsIContent* parent = aContainer;
+
+  // Handle appending of default content.
+  if (parent && parent->IsActiveChildrenElement()) {
+    XBLChildrenElement* childrenEl = static_cast<XBLChildrenElement*>(parent);
+    if (childrenEl->HasInsertedChildren()) {
+      // Removing default content that isn't being used. Ignore.
+      return;
+    }
+
+    parent = childrenEl->GetParent();
+  }
+
   do {
     nsXBLBinding* binding = GetBindingWithContent(parent);
     if (!binding) {
@@ -1064,6 +1099,19 @@ nsBindingManager::HandleChildInsertion(nsIContent* aContainer,
 
   XBLChildrenElement* point = nullptr;
   nsIContent* parent = aContainer;
+
+  // Handle insertion of default content.
+  if (parent && parent->IsActiveChildrenElement()) {
+    XBLChildrenElement* childrenEl = static_cast<XBLChildrenElement*>(parent);
+    if (childrenEl->HasInsertedChildren()) {
+      // Inserting default content that isn't being used. Ignore.
+      return;
+    }
+
+    childrenEl->MaybeSetupDefaultContent();
+    parent = childrenEl->GetParent();
+  }
+
   while (parent) {
     nsXBLBinding* binding = GetBindingWithContent(parent);
     if (!binding) {
@@ -1079,15 +1127,15 @@ nsBindingManager::HandleChildInsertion(nsIContent* aContainer,
     // TODO If there were multiple insertion points, this approximation can be
     // wrong. We need to re-run the distribution algorithm. In the meantime,
     // this should work well enough.
-    uint32_t index = aAppend ? point->mInsertedChildren.Length() : 0;
+    uint32_t index = aAppend ? point->InsertedChildrenLength() : 0;
     for (nsIContent* currentSibling = aChild->GetPreviousSibling();
          currentSibling;
          currentSibling = currentSibling->GetPreviousSibling()) {
       // If we find one of our previous siblings in the insertion point, the
       // index following it is the correct insertion point. Otherwise, we guess
       // based on whether we're appending or inserting.
-      uint32_t pointIndex = point->IndexOfInsertedChild(currentSibling);
-      if (pointIndex != point->NoIndex) {
+      int32_t pointIndex = point->IndexOfInsertedChild(currentSibling);
+      if (pointIndex != -1) {
         index = pointIndex + 1;
         break;
       }
