@@ -18,17 +18,16 @@
 class txXPathObjectAdaptor : public txIXPathObject
 {
 public:
-    txXPathObjectAdaptor(txAExprResult *aValue) : mValue(aValue)
+    explicit txXPathObjectAdaptor(txAExprResult* aValue) : mValue(aValue)
     {
         NS_ASSERTION(aValue,
                      "Don't create a txXPathObjectAdaptor if you don't have a "
                      "txAExprResult");
     }
-    virtual ~txXPathObjectAdaptor() {}
 
     NS_DECL_ISUPPORTS
 
-    NS_IMETHODIMP_(txAExprResult*) GetResult()
+    NS_IMETHODIMP_(txAExprResult*) GetResult() override
     {
         return mValue;
     }
@@ -38,6 +37,8 @@ protected:
     {
     }
 
+    virtual ~txXPathObjectAdaptor() {}
+
     nsRefPtr<txAExprResult> mValue;
 };
 
