@@ -9,8 +9,8 @@
 #ifndef js_Class_h
 #define js_Class_h
 
-#include "mozilla/NullPtr.h"
- 
+#include "mozilla/DebugOnly.h"
+
 #include "jstypes.h"
 
 #include "js/CallArgs.h"
@@ -24,12 +24,12 @@
  * object behavior and, e.g., allows custom slow layout.
  */
 
-class JSFreeOp;
+struct JSFreeOp;
 struct JSFunctionSpec;
 
 namespace js {
 
-class Class;
+struct Class;
 class FreeOp;
 class PropertyName;
 class Shape;
@@ -40,6 +40,12 @@ extern JS_FRIEND_DATA(const js::Class* const) FunctionClassPtr;
 
 } // namespace js
 
+namespace JS {
+
+class AutoIdVector;
+
+}
+
 // JSClass operation signatures.
 
 // Add or get a property named by id in obj.  Note the jsid id type -- id may
@@ -75,39 +81,19 @@ typedef bool
 (* JSDeletePropertyOp)(JSContext* cx, JS::HandleObject obj, JS::HandleId id,
                        bool* succeeded);
 
-// This function type is used for callbacks that enumerate the properties of
-// a JSObject.  The behavior depends on the value of enum_op:
+// The type of ObjectOps::enumerate. This callback overrides a portion of SpiderMonkey's default
+// [[Enumerate]] internal method. When an ordinary object is enumerated, that object and each object
+// on its prototype chain is tested for an enumerate op, and those ops are called in order.
+// The properties each op adds to the 'properties' vector are added to the set of values the
+// for-in loop will iterate over. All of this is nonstandard.
 //
-//  JSENUMERATE_INIT
-//    A new, opaque iterator state should be allocated and stored in *statep.
-//    (You can use PRIVATE_TO_JSVAL() to tag the pointer to be stored).
+// An object is "enumerated" when it's the target of a for-in loop or JS_Enumerate().
+// All other property inspection, including Object.keys(obj), goes through [[OwnKeys]].
 //
-//    The number of properties that will be enumerated should be returned as
-//    an integer jsval in *idp, if idp is non-null, and provided the number of
-//    enumerable properties is known.  If idp is non-null and the number of
-//    enumerable properties can't be computed in advance, *idp should be set
-//    to JSVAL_ZERO.
-//
-//  JSENUMERATE_INIT_ALL
-//    Used identically to JSENUMERATE_INIT, but exposes all properties of the
-//    object regardless of enumerability.
-//
-//  JSENUMERATE_NEXT
-//    A previously allocated opaque iterator state is passed in via statep.
-//    Return the next jsid in the iteration using *idp.  The opaque iterator
-//    state pointed at by statep is destroyed and *statep is set to JSVAL_NULL
-//    if there are no properties left to enumerate.
-//
-//  JSENUMERATE_DESTROY
-//    Destroy the opaque iterator state previously allocated in *statep by a
-//    call to this function when enum_op was JSENUMERATE_INIT or
-//    JSENUMERATE_INIT_ALL.
-//
-// The return value is used to indicate success, with a value of false
-// indicating failure.
+// The callback's job is to populate 'properties' with all property keys that the for-in loop
+// should visit.
 typedef bool
-(* JSNewEnumerateOp)(JSContext* cx, JS::HandleObject obj, JSIterateOp enum_op,
-                     JS::MutableHandleValue statep, JS::MutableHandleId idp);
+(* JSNewEnumerateOp)(JSContext* cx, JS::HandleObject obj, JS::AutoIdVector& properties);
 
 // The old-style JSClass.enumerate op should define all lazy properties not
 // yet reflected in obj.
@@ -119,24 +105,12 @@ typedef bool
 // (e.g., the DOM attributes for a given node reflected as obj) on demand.
 //
 // JS looks for a property in an object, and if not found, tries to resolve
-// the given id.  If resolve succeeds, the engine looks again in case resolve
-// defined obj[id].  If no such property exists directly in obj, the process
-// is repeated with obj's prototype, etc.
+// the given id. *resolvedp should be set to true iff the property was
+// was defined on |obj|.
 //
-// NB: JSNewResolveOp provides a cheaper way to resolve lazy properties.
 typedef bool
-(* JSResolveOp)(JSContext* cx, JS::HandleObject obj, JS::HandleId id);
-
-// Like JSResolveOp, except the *objp out parameter, on success, should be null
-// to indicate that id was not resolved; and non-null, referring to obj or one
-// of its prototypes, if id was resolved.  The hook may assume* objp is null on
-// entry.
-//
-// This hook instead of JSResolveOp is called via the JSClass.resolve member
-// if JSCLASS_NEW_RESOLVE is set in JSClass.flags.
-typedef bool
-(* JSNewResolveOp)(JSContext* cx, JS::HandleObject obj, JS::HandleId id,
-                   JS::MutableHandleObject objp);
+(* JSResolveOp)(JSContext* cx, JS::HandleObject obj, JS::HandleId id,
+                bool* resolvedp);
 
 // Convert obj to the given type, returning true with the resulting value in
 // *vp on success, and returning false on error or exception.
@@ -152,7 +126,7 @@ typedef void
 
 // Finalizes external strings created by JS_NewExternalString.
 struct JSStringFinalizer {
-    void (*finalize)(const JSStringFinalizer* fin, jschar* chars);
+    void (*finalize)(const JSStringFinalizer* fin, char16_t* chars);
 };
 
 // Check whether v is an instance of obj.  Return false on error or exception,
@@ -177,70 +151,35 @@ typedef bool
 typedef void
 (* JSTraceOp)(JSTracer* trc, JSObject* obj);
 
-// A generic type for functions mapping an object to another object, or null
-// if an error or exception was thrown on cx.
-typedef JSObject*
-(* JSObjectOp)(JSContext* cx, JS::HandleObject obj);
-
-// Hook that creates an iterator object for a given object. Returns the
-// iterator object or null if an error or exception was thrown on cx.
-typedef JSObject*
-(* JSIteratorOp)(JSContext* cx, JS::HandleObject obj, bool keysonly);
-
 typedef JSObject*
 (* JSWeakmapKeyDelegateOp)(JSObject* obj);
 
+typedef void
+(* JSObjectMovedOp)(JSObject* obj, const JSObject* old);
+
 /* js::Class operation signatures. */
 
 namespace js {
 
 typedef bool
-(* LookupGenericOp)(JSContext* cx, JS::HandleObject obj, JS::HandleId id,
-                    JS::MutableHandleObject objp, JS::MutableHandle<Shape*> propp);
+(* LookupPropertyOp)(JSContext* cx, JS::HandleObject obj, JS::HandleId id,
+                     JS::MutableHandleObject objp, JS::MutableHandle<Shape*> propp);
 typedef bool
-(* LookupPropOp)(JSContext* cx, JS::HandleObject obj, JS::Handle<PropertyName*> name,
-                 JS::MutableHandleObject objp, JS::MutableHandle<Shape*> propp);
+(* DefinePropertyOp)(JSContext* cx, JS::HandleObject obj, JS::HandleId id, JS::HandleValue value,
+                     JSPropertyOp getter, JSStrictPropertyOp setter, unsigned attrs);
 typedef bool
-(* LookupElementOp)(JSContext* cx, JS::HandleObject obj, uint32_t index,
-                    JS::MutableHandleObject objp, JS::MutableHandle<Shape*> propp);
+(* HasPropertyOp)(JSContext* cx, JS::HandleObject obj, JS::HandleId id, bool* foundp);
 typedef bool
-(* DefineGenericOp)(JSContext* cx, JS::HandleObject obj, JS::HandleId id, JS::HandleValue value,
-                    JSPropertyOp getter, JSStrictPropertyOp setter, unsigned attrs);
+(* GetPropertyOp)(JSContext* cx, JS::HandleObject obj, JS::HandleObject receiver, JS::HandleId id,
+                  JS::MutableHandleValue vp);
 typedef bool
-(* DefinePropOp)(JSContext* cx, JS::HandleObject obj, JS::Handle<PropertyName*> name,
-                 JS::HandleValue value, JSPropertyOp getter, JSStrictPropertyOp setter,
-                 unsigned attrs);
+(* SetPropertyOp)(JSContext* cx, JS::HandleObject obj, JS::HandleObject receiver, JS::HandleId id,
+                  JS::MutableHandleValue vp, bool strict);
 typedef bool
-(* DefineElementOp)(JSContext* cx, JS::HandleObject obj, uint32_t index, JS::HandleValue value,
-                    JSPropertyOp getter, JSStrictPropertyOp setter, unsigned attrs);
+(* GetOwnPropertyOp)(JSContext* cx, JS::HandleObject obj, JS::HandleId id,
+                     JS::MutableHandle<JSPropertyDescriptor> desc);
 typedef bool
-(* GenericIdOp)(JSContext* cx, JS::HandleObject obj, JS::HandleObject receiver, JS::HandleId id,
-                JS::MutableHandleValue vp);
-typedef bool
-(* PropertyIdOp)(JSContext* cx, JS::HandleObject obj, JS::HandleObject receiver,
-                 JS::Handle<PropertyName*> name, JS::MutableHandleValue vp);
-typedef bool
-(* ElementIdOp)(JSContext* cx, JS::HandleObject obj, JS::HandleObject receiver, uint32_t index,
-                JS::MutableHandleValue vp);
-typedef bool
-(* StrictGenericIdOp)(JSContext* cx, JS::HandleObject obj, JS::HandleId id,
-                      JS::MutableHandleValue vp, bool strict);
-typedef bool
-(* StrictPropertyIdOp)(JSContext* cx, JS::HandleObject obj, JS::Handle<PropertyName*> name,
-                       JS::MutableHandleValue vp, bool strict);
-typedef bool
-(* StrictElementIdOp)(JSContext* cx, JS::HandleObject obj, uint32_t index,
-                      JS::MutableHandleValue vp, bool strict);
-typedef bool
-(* GenericAttributesOp)(JSContext* cx, JS::HandleObject obj, JS::HandleId id, unsigned* attrsp);
-typedef bool
-(* PropertyAttributesOp)(JSContext* cx, JS::HandleObject obj, JS::Handle<PropertyName*> name,
-                         unsigned* attrsp);
-typedef bool
-(* DeletePropertyOp)(JSContext* cx, JS::HandleObject obj, JS::Handle<PropertyName*> name,
-                     bool* succeeded);
-typedef bool
-(* DeleteElementOp)(JSContext* cx, JS::HandleObject obj, uint32_t index, bool* succeeded);
+(* DeletePropertyOp)(JSContext* cx, JS::HandleObject obj, JS::HandleId id, bool* succeeded);
 
 typedef bool
 (* WatchOp)(JSContext* cx, JS::HandleObject obj, JS::HandleId id, JS::HandleObject callable);
@@ -248,12 +187,54 @@ typedef bool
 typedef bool
 (* UnwatchOp)(JSContext* cx, JS::HandleObject obj, JS::HandleId id);
 
-typedef bool
-(* SliceOp)(JSContext* cx, JS::HandleObject obj, uint32_t begin, uint32_t end,
-            JS::HandleObject result); // result is actually preallocted.
+class JS_FRIEND_API(ElementAdder)
+{
+  public:
+    enum GetBehavior {
+        // Check if the element exists before performing the Get and preserve
+        // holes.
+        CheckHasElemPreserveHoles,
 
+        // Perform a Get operation, like obj[index] in JS.
+        GetElement
+    };
+
+  private:
+    // Only one of these is used.
+    JS::RootedObject resObj_;
+    JS::Value* vp_;
+
+    uint32_t index_;
+    mozilla::DebugOnly<uint32_t> length_;
+    GetBehavior getBehavior_;
+
+  public:
+    ElementAdder(JSContext* cx, JSObject* obj, uint32_t length, GetBehavior behavior)
+      : resObj_(cx, obj), vp_(nullptr), index_(0), length_(length), getBehavior_(behavior)
+    {}
+    ElementAdder(JSContext* cx, JS::Value* vp, uint32_t length, GetBehavior behavior)
+      : resObj_(cx), vp_(vp), index_(0), length_(length), getBehavior_(behavior)
+    {}
+
+    GetBehavior getBehavior() const { return getBehavior_; }
+
+    void append(JSContext* cx, JS::HandleValue v);
+    void appendHole();
+};
+
+typedef bool
+(* GetElementsOp)(JSContext* cx, JS::HandleObject obj, uint32_t begin, uint32_t end,
+                  ElementAdder* adder);
+
+// A generic type for functions mapping an object to another object, or null
+// if an error or exception was thrown on cx.
 typedef JSObject*
 (* ObjectOp)(JSContext* cx, JS::HandleObject obj);
+
+// Hook to map an object to its inner object. Infallible.
+typedef JSObject*
+(* InnerObjectOp)(JSObject* obj);
+
 typedef void
 (* FinalizeOp)(FreeOp* fop, JSObject* obj);
 
@@ -261,7 +242,7 @@ typedef void
     const char*         name;                                                \
     uint32_t            flags;                                                \
                                                                               \
-    /* Mandatory function pointer members. */                                 \
+    /* Function pointer members (may be null). */                             \
     JSPropertyOp        addProperty;                                          \
     JSDeletePropertyOp  delProperty;                                          \
     JSPropertyOp        getProperty;                                          \
@@ -269,8 +250,6 @@ typedef void
     JSEnumerateOp       enumerate;                                            \
     JSResolveOp         resolve;                                              \
     JSConvertOp         convert;                                              \
-                                                                              \
-    /* Optional members (may be null). */                                     \
     FinalizeOpType      finalize;                                             \
     JSNative            call;                                                 \
     JSHasInstanceOp     hasInstance;                                          \
@@ -284,21 +263,45 @@ typedef JSObject* (*ClassObjectCreationOp)(JSContext* cx, JSProtoKey key);
 typedef bool (*FinishClassInitOp)(JSContext* cx, JS::HandleObject ctor,
                                   JS::HandleObject proto);
 
+const size_t JSCLASS_CACHED_PROTO_WIDTH = 6;
+
 struct ClassSpec
 {
     ClassObjectCreationOp createConstructor;
     ClassObjectCreationOp createPrototype;
     const JSFunctionSpec* constructorFunctions;
     const JSFunctionSpec* prototypeFunctions;
+    const JSPropertySpec* prototypeProperties;
     FinishClassInitOp finishInit;
+    uintptr_t flags;
+
+    static const size_t ParentKeyWidth = JSCLASS_CACHED_PROTO_WIDTH;
+
+    static const uintptr_t ParentKeyMask = (1 << ParentKeyWidth) - 1;
+    static const uintptr_t DontDefineConstructor = 1 << ParentKeyWidth;
+
     bool defined() const { return !!createConstructor; }
+
+    bool dependent() const {
+        MOZ_ASSERT(defined());
+        return (flags & ParentKeyMask);
+    }
+
+    JSProtoKey parentKey() const {
+        static_assert(JSProto_Null == 0, "zeroed key must be null");
+        return JSProtoKey(flags & ParentKeyMask);
+    }
+
+    bool shouldDefineConstructor() const {
+        MOZ_ASSERT(defined());
+        return !(flags & DontDefineConstructor);
+    }
 };
 
 struct ClassExtension
 {
-    JSObjectOp          outerObject;
-    JSObjectOp          innerObject;
-    JSIteratorOp        iteratorObject;
+    ObjectOp            outerObject;
+    InnerObjectOp       innerObject;
 
     /*
      * isWrappedNative is true only if the class is an XPCWrappedNative.
@@ -318,41 +321,43 @@ struct ClassExtension
      * wrapped object is collected.
      */
     JSWeakmapKeyDelegateOp weakmapKeyDelegateOp;
+
+    /*
+     * Optional hook called when an object is moved by a compacting GC.
+     *
+     * There may exist weak pointers to an object that are not traced through
+     * when the normal trace APIs are used, for example objects in the wrapper
+     * cache. This hook allows these pointers to be updated.
+     *
+     * Note that this hook can be called before JS_NewObject() returns if a GC
+     * is triggered during construction of the object. This can happen for
+     * global objects for example.
+     */
+    JSObjectMovedOp objectMovedOp;
 };
 
-#define JS_NULL_CLASS_SPEC  {nullptr,nullptr,nullptr,nullptr,nullptr}
-#define JS_NULL_CLASS_EXT   {nullptr,nullptr,nullptr,false,nullptr}
+#define JS_NULL_CLASS_SPEC  {nullptr,nullptr,nullptr,nullptr,nullptr,nullptr}
+#define JS_NULL_CLASS_EXT   {nullptr,nullptr,false,nullptr,nullptr}
 
 struct ObjectOps
 {
-    LookupGenericOp     lookupGeneric;
-    LookupPropOp        lookupProperty;
-    LookupElementOp     lookupElement;
-    DefineGenericOp     defineGeneric;
-    DefinePropOp        defineProperty;
-    DefineElementOp     defineElement;
-    GenericIdOp         getGeneric;
-    PropertyIdOp        getProperty;
-    ElementIdOp         getElement;
-    StrictGenericIdOp   setGeneric;
-    StrictPropertyIdOp  setProperty;
-    StrictElementIdOp   setElement;
-    GenericAttributesOp getGenericAttributes;
-    GenericAttributesOp setGenericAttributes;
+    LookupPropertyOp    lookupProperty;
+    DefinePropertyOp    defineProperty;
+    HasPropertyOp       hasProperty;
+    GetPropertyOp       getProperty;
+    SetPropertyOp       setProperty;
+    GetOwnPropertyOp    getOwnPropertyDescriptor;
     DeletePropertyOp    deleteProperty;
-    DeleteElementOp     deleteElement;
     WatchOp             watch;
     UnwatchOp           unwatch;
-    SliceOp             slice; // Optimized slice, can be null.
-
+    GetElementsOp       getElements;
     JSNewEnumerateOp    enumerate;
     ObjectOp            thisObject;
 };
 
 #define JS_NULL_OBJECT_OPS                                                    \
-    {nullptr,nullptr,nullptr,nullptr,nullptr,nullptr,nullptr,nullptr,nullptr, \
-     nullptr,nullptr,nullptr,nullptr,nullptr,nullptr,nullptr,nullptr,nullptr, \
-     nullptr,nullptr}
+    {nullptr, nullptr, nullptr, nullptr, nullptr, nullptr, nullptr, nullptr,  \
+     nullptr, nullptr, nullptr, nullptr}
 
 } // namespace js
 
@@ -363,12 +368,10 @@ typedef void (*JSClassInternal)();
 struct JSClass {
     JS_CLASS_MEMBERS(JSFinalizeOp);
 
-    void*               reserved[31];
+    void*               reserved[24];
 };
 
 #define JSCLASS_HAS_PRIVATE             (1<<0)  // objects have private slot
-#define JSCLASS_NEW_ENUMERATE           (1<<1)  // has JSNewEnumerateOp hook
-#define JSCLASS_NEW_RESOLVE             (1<<2)  // has JSNewResolveOp hook
 #define JSCLASS_PRIVATE_IS_NSISUPPORTS  (1<<3)  // private is (nsISupports*)
 #define JSCLASS_IS_DOMJSCLASS           (1<<4)  // objects are DOM
 #define JSCLASS_IMPLEMENTS_BARRIERS     (1<<5)  // Correctly implements GC read
@@ -400,7 +403,7 @@ struct JSClass {
 
 #define JSCLASS_IS_PROXY                (1<<(JSCLASS_HIGH_FLAGS_SHIFT+4))
 
-// Bit 22 unused.
+#define JSCLASS_FINALIZE_FROM_NURSERY   (1<<(JSCLASS_HIGH_FLAGS_SHIFT+5))
 
 // Reserved for embeddings.
 #define JSCLASS_USERBIT2                (1<<(JSCLASS_HIGH_FLAGS_SHIFT+6))
@@ -421,7 +424,11 @@ struct JSClass {
 // with the following flags. Failure to use JSCLASS_GLOBAL_FLAGS was
 // previously allowed, but is now an ES5 violation and thus unsupported.
 //
-#define JSCLASS_GLOBAL_SLOT_COUNT      (3 + JSProto_LIMIT * 3 + 31)
+// JSCLASS_GLOBAL_APPLICATION_SLOTS is the number of slots reserved at
+// the beginning of every global object's slots for use by the
+// application.
+#define JSCLASS_GLOBAL_APPLICATION_SLOTS 4
+#define JSCLASS_GLOBAL_SLOT_COUNT      (JSCLASS_GLOBAL_APPLICATION_SLOTS + JSProto_LIMIT * 3 + 31)
 #define JSCLASS_GLOBAL_FLAGS_WITH_SLOTS(n)                                    \
     (JSCLASS_IS_GLOBAL | JSCLASS_HAS_RESERVED_SLOTS(JSCLASS_GLOBAL_SLOT_COUNT + (n)))
 #define JSCLASS_GLOBAL_FLAGS                                                  \
@@ -432,7 +439,6 @@ struct JSClass {
 
 // Fast access to the original value of each standard class's prototype.
 #define JSCLASS_CACHED_PROTO_SHIFT      (JSCLASS_HIGH_FLAGS_SHIFT + 10)
-#define JSCLASS_CACHED_PROTO_WIDTH      6
 #define JSCLASS_CACHED_PROTO_MASK       JS_BITMASK(JSCLASS_CACHED_PROTO_WIDTH)
 #define JSCLASS_HAS_CACHED_PROTO(key)   (uint32_t(key) << JSCLASS_CACHED_PROTO_SHIFT)
 #define JSCLASS_CACHED_PROTO_KEY(clasp) ((JSProtoKey)                         \
@@ -453,7 +459,13 @@ struct Class
     ClassExtension      ext;
     ObjectOps           ops;
 
-    /* Class is not native and its map is not a scope. */
+    /*
+     * Objects of this class aren't native objects. They don't have Shapes that
+     * describe their properties and layout. Classes using this flag must
+     * provide their own property behavior, either by being proxy classes (do
+     * this) or by overriding all the ObjectOps except getElements, watch,
+     * unwatch, and thisObject (don't do this).
+     */
     static const uint32_t NON_NATIVE = JSCLASS_INTERNAL_FLAG2;
 
     bool isNative() const {
@@ -472,7 +484,8 @@ struct Class
         return this == js::FunctionClassPtr;
     }
 
-    bool isCallable() const {
+    bool nonProxyCallable() const {
+        MOZ_ASSERT(!isProxy());
         return isJSFunction() || call;
     }
 
@@ -487,21 +500,36 @@ struct Class
     static size_t offsetOfFlags() { return offsetof(Class, flags); }
 };
 
-JS_STATIC_ASSERT(offsetof(JSClass, name) == offsetof(Class, name));
-JS_STATIC_ASSERT(offsetof(JSClass, flags) == offsetof(Class, flags));
-JS_STATIC_ASSERT(offsetof(JSClass, addProperty) == offsetof(Class, addProperty));
-JS_STATIC_ASSERT(offsetof(JSClass, delProperty) == offsetof(Class, delProperty));
-JS_STATIC_ASSERT(offsetof(JSClass, getProperty) == offsetof(Class, getProperty));
-JS_STATIC_ASSERT(offsetof(JSClass, setProperty) == offsetof(Class, setProperty));
-JS_STATIC_ASSERT(offsetof(JSClass, enumerate) == offsetof(Class, enumerate));
-JS_STATIC_ASSERT(offsetof(JSClass, resolve) == offsetof(Class, resolve));
-JS_STATIC_ASSERT(offsetof(JSClass, convert) == offsetof(Class, convert));
-JS_STATIC_ASSERT(offsetof(JSClass, finalize) == offsetof(Class, finalize));
-JS_STATIC_ASSERT(offsetof(JSClass, call) == offsetof(Class, call));
-JS_STATIC_ASSERT(offsetof(JSClass, construct) == offsetof(Class, construct));
-JS_STATIC_ASSERT(offsetof(JSClass, hasInstance) == offsetof(Class, hasInstance));
-JS_STATIC_ASSERT(offsetof(JSClass, trace) == offsetof(Class, trace));
-JS_STATIC_ASSERT(sizeof(JSClass) == sizeof(Class));
+static_assert(offsetof(JSClass, name) == offsetof(Class, name),
+              "Class and JSClass must be consistent");
+static_assert(offsetof(JSClass, flags) == offsetof(Class, flags),
+              "Class and JSClass must be consistent");
+static_assert(offsetof(JSClass, addProperty) == offsetof(Class, addProperty),
+              "Class and JSClass must be consistent");
+static_assert(offsetof(JSClass, delProperty) == offsetof(Class, delProperty),
+              "Class and JSClass must be consistent");
+static_assert(offsetof(JSClass, getProperty) == offsetof(Class, getProperty),
+              "Class and JSClass must be consistent");
+static_assert(offsetof(JSClass, setProperty) == offsetof(Class, setProperty),
+              "Class and JSClass must be consistent");
+static_assert(offsetof(JSClass, enumerate) == offsetof(Class, enumerate),
+              "Class and JSClass must be consistent");
+static_assert(offsetof(JSClass, resolve) == offsetof(Class, resolve),
+              "Class and JSClass must be consistent");
+static_assert(offsetof(JSClass, convert) == offsetof(Class, convert),
+              "Class and JSClass must be consistent");
+static_assert(offsetof(JSClass, finalize) == offsetof(Class, finalize),
+              "Class and JSClass must be consistent");
+static_assert(offsetof(JSClass, call) == offsetof(Class, call),
+              "Class and JSClass must be consistent");
+static_assert(offsetof(JSClass, construct) == offsetof(Class, construct),
+              "Class and JSClass must be consistent");
+static_assert(offsetof(JSClass, hasInstance) == offsetof(Class, hasInstance),
+              "Class and JSClass must be consistent");
+static_assert(offsetof(JSClass, trace) == offsetof(Class, trace),
+              "Class and JSClass must be consistent");
+static_assert(sizeof(JSClass) == sizeof(Class),
+              "Class and JSClass must be consistent");
 
 static MOZ_ALWAYS_INLINE const JSClass*
 Jsvalify(const Class* c)
@@ -520,8 +548,10 @@ Valueify(const JSClass* c)
  * value of objects.
  */
 enum ESClassValue {
-    ESClass_Array, ESClass_Number, ESClass_String, ESClass_Boolean,
-    ESClass_RegExp, ESClass_ArrayBuffer, ESClass_Date,
+    ESClass_Object, ESClass_Array, ESClass_Number, ESClass_String,
+    ESClass_Boolean, ESClass_RegExp, ESClass_ArrayBuffer, ESClass_SharedArrayBuffer,
+    ESClass_Date, ESClass_Set, ESClass_Map,
+
     // Special snowflake for the ES6 IsArray method.
     // Please don't use it without calling that function.
     ESClass_IsArray
@@ -540,6 +570,15 @@ ObjectClassIs(JSObject& obj, ESClassValue classValue, JSContext* cx);
 inline bool
 IsObjectWithClass(const JS::Value& v, ESClassValue classValue, JSContext* cx);
 
+/* Fills |vp| with the unboxed value for boxed types, or undefined otherwise. */
+inline bool
+Unbox(JSContext* cx, JS::HandleObject obj, JS::MutableHandleValue vp);
+
+#ifdef DEBUG
+JS_FRIEND_API(bool)
+HasObjectMovedOp(JSObject* obj);
+#endif
+
 }  /* namespace js */
 
 #endif  /* js_Class_h */
