@@ -7,9 +7,7 @@
 #ifndef jit_x64_BaselineRegisters_x64_h
 #define jit_x64_BaselineRegisters_x64_h
 
-#ifdef JS_ION
-
-#include "jit/IonMacroAssembler.h"
+#include "jit/MacroAssembler.h"
 
 namespace js {
 namespace jit {
@@ -34,6 +32,4 @@ static MOZ_CONSTEXPR_VAR FloatRegister FloatReg1      = xmm1;
 } // namespace jit
 } // namespace js
 
-#endif // JS_ION
-
 #endif /* jit_x64_BaselineRegisters_x64_h */
