@@ -1,4 +1,4 @@
-/* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
+/* -*- indent-tabs-mode: nil; js-indent-level: 2 -*- */
 
 /*
  * Any copyright is dedicated to the Public Domain.
@@ -6,22 +6,22 @@
  */
 
 /*
- * Prefix decrement expressions must not have 'eval' or 'arguments' as
- * their operands.
+ * Postfix increment expressions must not have 'eval' or 'arguments'
+ * as their operands.
  */
-assertEq(testLenientAndStrict('--arguments',
+assertEq(testLenientAndStrict('arguments++',
                               parsesSuccessfully,
                               parseRaisesException(SyntaxError)),
          true);
-assertEq(testLenientAndStrict('--eval',
+assertEq(testLenientAndStrict('eval++',
                               parsesSuccessfully,
                               parseRaisesException(SyntaxError)),
          true);
-assertEq(testLenientAndStrict('--(arguments)',
+assertEq(testLenientAndStrict('(arguments)++',
                               parsesSuccessfully,
                               parseRaisesException(SyntaxError)),
          true);
-assertEq(testLenientAndStrict('--(eval)',
+assertEq(testLenientAndStrict('(eval)++',
                               parsesSuccessfully,
                               parseRaisesException(SyntaxError)),
          true);
