@@ -1,11 +1,11 @@
-/* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
+/* -*- indent-tabs-mode: nil; js-indent-level: 2 -*- */
 /* This Source Code Form is subject to the terms of the Mozilla Public
  * License, v. 2.0. If a copy of the MPL was not distributed with this
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
 //-----------------------------------------------------------------------------
-var BUGNUMBER = 467495;
-var summary = 'TCF_FUN_CLOSURE_VS_VAR is necessary';
+var BUGNUMBER = 466905;
+var summary = 'Do not assert: v_ins->isCall() && v_ins->callInfo() == &js_FastNewArray_ci';
 var actual = '';
 var expect = '';
 
@@ -19,22 +19,10 @@ function test()
   enterFunc ('test');
   printBugNumber(BUGNUMBER);
   printStatus (summary);
-
-  function g() {
-    if (1)
-      function x() {};
-    var x;
-    const y = 1;
-  }
-
-  try
-  {
-    g();
-  }
-  catch(ex)
-  {
-    actual = ex + '';
-  }
+ 
+  function f(a) { for each (let c in a) [(c > 5) ? 'A' : 'B']; }
+  f([true, 8]);
+  f([2]);
 
   reportCompare(expect, actual, summary);
 
