@@ -4,20 +4,21 @@
 /// "Use Terms").   Any redistribution of this code must retain the above 
 /// copyright and this notice and otherwise comply with the Use Terms.
 /**
- * @path ch11/11.13/11.13.2/11.13.2-6-20-s.js
- * @description Strict Mode - SyntaxError is thrown if the identifier arguments appear as the LeftHandSideExpression of a Compound Assignment operator(&=)
+ * @path ch11/11.4/11.4.1/11.4.1-5-a-10-s.js
+ * @description Strict Mode - SyntaxError is thrown when deleting a variable of type Array
  * @onlyStrict
  */
 
 
 function testcase() {
         "use strict";
-        var blah = arguments;
+        var arrObj = [1,2,3];
+
         try {
-            eval("arguments &= 20;");
+            eval("delete arrObj;");
             return false;
         } catch (e) {
-            return e instanceof SyntaxError && blah === arguments;
+            return e instanceof SyntaxError;
         }
     }
 runTestCase(testcase);
