@@ -4,8 +4,8 @@
 /// "Use Terms").   Any redistribution of this code must retain the above 
 /// copyright and this notice and otherwise comply with the Use Terms.
 /**
- * @path ch12/12.10/12.10.1/12.10.1-15-s.js
- * @description Strict Mode - SyntaxError is thrown when the RHS of a dot property assignment utilizes WithStatement
+ * @path ch12/12.14/12.14.1/12.14.1-2-s.js
+ * @description Strict Mode - SyntaxError is thrown if a TryStatement with a Catch occurs within strict code and the Identifier of the Catch production is arguments
  * @onlyStrict
  */
 
@@ -14,7 +14,9 @@ function testcase() {
         "use strict";
 
         try {
-            eval("var obj = {}; obj.get = function (a) { with(a){} }; ");
+            eval("\
+                   try {} catch (arguments) { }\
+            ");
             return false;
         } catch (e) {
             return e instanceof SyntaxError;
