@@ -15,30 +15,29 @@ namespace xpc {
 
 class WaiveXrayWrapper : public js::CrossCompartmentWrapper {
   public:
-    WaiveXrayWrapper(unsigned flags);
-    virtual ~WaiveXrayWrapper();
+    explicit MOZ_CONSTEXPR WaiveXrayWrapper(unsigned flags) : js::CrossCompartmentWrapper(flags) { }
 
-    virtual bool getPropertyDescriptor(JSContext* cx, JS::Handle<JSObject*> wrapper,
-                                       JS::Handle<jsid> id,
-                                       JS::MutableHandle<JSPropertyDescriptor> desc) MOZ_OVERRIDE;
     virtual bool getOwnPropertyDescriptor(JSContext* cx, JS::Handle<JSObject*> wrapper,
                                           JS::Handle<jsid> id,
-                                          JS::MutableHandle<JSPropertyDescriptor> desc) MOZ_OVERRIDE;
-    virtual bool get(JSContext* cx, JS::Handle<JSObject*> wrapper, JS::Handle<JSObject*> receiver,
-                     JS::Handle<jsid> id, JS::MutableHandle<JS::Value> vp) MOZ_OVERRIDE;
-
-    virtual bool call(JSContext* cx, JS::Handle<JSObject*> wrapper,
-                      const JS::CallArgs& args) MOZ_OVERRIDE;
-    virtual bool construct(JSContext* cx, JS::Handle<JSObject*> wrapper,
-                           const JS::CallArgs& args) MOZ_OVERRIDE;
-
-    virtual bool nativeCall(JSContext* cx, JS::IsAcceptableThis test,
-                            JS::NativeImpl impl, JS::CallArgs args) MOZ_OVERRIDE;
-
+                                          JS::MutableHandle<JSPropertyDescriptor> desc) const override;
     virtual bool getPrototypeOf(JSContext* cx, JS::Handle<JSObject*> wrapper,
-                                JS::MutableHandle<JSObject*> protop) MOZ_OVERRIDE;
+                                JS::MutableHandle<JSObject*> protop) const override;
+    virtual bool get(JSContext* cx, JS::Handle<JSObject*> wrapper, JS::Handle<JSObject*> receiver,
+                     JS::Handle<jsid> id, JS::MutableHandle<JS::Value> vp) const override;
+    virtual bool call(JSContext* cx, JS::Handle<JSObject*> wrapper,
+                      const JS::CallArgs& args) const override;
+    virtual bool construct(JSContext* cx, JS::Handle<JSObject*> wrapper,
+                           const JS::CallArgs& args) const override;
 
-    static WaiveXrayWrapper singleton;
+    virtual bool enumerate(JSContext* cx, JS::Handle<JSObject*> proxy,
+                           JS::MutableHandle<JSObject*> objp) const override;
+    virtual bool nativeCall(JSContext* cx, JS::IsAcceptableThis test,
+                            JS::NativeImpl impl, JS::CallArgs args) const override;
+    virtual bool getPropertyDescriptor(JSContext* cx, JS::Handle<JSObject*> wrapper,
+                                       JS::Handle<jsid> id,
+                                       JS::MutableHandle<JSPropertyDescriptor> desc) const override;
+
+    static const WaiveXrayWrapper singleton;
 };
 
 }
