@@ -9,71 +9,85 @@
 #include "nsBlockReflowContext.h"
 #include "nsBlockReflowState.h"
 #include "nsFloatManager.h"
+#include "nsColumnSetFrame.h"
 #include "nsContainerFrame.h"
 #include "nsBlockFrame.h"
 #include "nsLineBox.h"
 #include "nsLayoutUtils.h"
 
+using namespace mozilla;
+
 #ifdef DEBUG
 #undef  NOISY_MAX_ELEMENT_SIZE
 #undef   REALLY_NOISY_MAX_ELEMENT_SIZE
-#undef  NOISY_VERTICAL_MARGINS
+#undef  NOISY_BLOCK_DIR_MARGINS
 #else
 #undef  NOISY_MAX_ELEMENT_SIZE
 #undef   REALLY_NOISY_MAX_ELEMENT_SIZE
-#undef  NOISY_VERTICAL_MARGINS
+#undef  NOISY_BLOCK_DIR_MARGINS
 #endif
 
 nsBlockReflowContext::nsBlockReflowContext(nsPresContext* aPresContext,
                                            const nsHTMLReflowState& aParentRS)
   : mPresContext(aPresContext),
     mOuterReflowState(aParentRS),
-    mMetrics(aParentRS.GetWritingMode())
+    mSpace(aParentRS.GetWritingMode()),
+    mMetrics(aParentRS)
 {
 }
 
 static nsIFrame* DescendIntoBlockLevelFrame(nsIFrame* aFrame)
 {
   nsIAtom* type = aFrame->GetType();
-  if (type == nsGkAtoms::columnSetFrame)
-    return DescendIntoBlockLevelFrame(aFrame->GetFirstPrincipalChild());
+  if (type == nsGkAtoms::columnSetFrame) {
+    static_cast<nsColumnSetFrame*>(aFrame)->DrainOverflowColumns();
+    nsIFrame* child = aFrame->GetFirstPrincipalChild();
+    if (child) {
+      return DescendIntoBlockLevelFrame(child);
+    }
+  }
   return aFrame;
 }
 
 bool
-nsBlockReflowContext::ComputeCollapsedTopMargin(const nsHTMLReflowState& aRS,
-  nsCollapsingMargin* aMargin, nsIFrame* aClearanceFrame,
-  bool* aMayNeedRetry, bool* aBlockIsEmpty)
+nsBlockReflowContext::ComputeCollapsedBStartMargin(const nsHTMLReflowState& aRS,
+                                                   nsCollapsingMargin* aMargin,
+                                                   nsIFrame* aClearanceFrame,
+                                                   bool* aMayNeedRetry,
+                                                   bool* aBlockIsEmpty)
 {
-  // Include frame's top margin
-  aMargin->Include(aRS.ComputedPhysicalMargin().top);
+  WritingMode wm = aRS.GetWritingMode();
+  WritingMode parentWM = mMetrics.GetWritingMode();
 
-  // The inclusion of the bottom margin when empty is done by the caller
+  // Include block-start element of frame's margin
+  aMargin->Include(aRS.ComputedLogicalMargin().ConvertTo(parentWM, wm).BStart(parentWM));
+
+  // The inclusion of the block-end margin when empty is done by the caller
   // since it doesn't need to be done by the top-level (non-recursive)
   // caller.
 
-#ifdef NOISY_VERTICAL_MARGINS
+#ifdef NOISY_BLOCKDIR_MARGINS
   nsFrame::ListTag(stdout, aRS.frame);
-  printf(": %d => %d\n", aRS.ComputedPhysicalMargin().top, aMargin->get());
+  printf(": %d => %d\n", aRS.ComputedLogicalMargin().BStart(wm), aMargin->get());
 #endif
 
   bool dirtiedLine = false;
   bool setBlockIsEmpty = false;
 
-  // Calculate the frame's generational top-margin from its child
-  // blocks. Note that if the frame has a non-zero top-border or
-  // top-padding then this step is skipped because it will be a margin
+  // Calculate the frame's generational block-start-margin from its child
+  // blocks. Note that if the frame has a non-zero block-start-border or
+  // block-start-padding then this step is skipped because it will be a margin
   // root.  It is also skipped if the frame is a margin root for other
   // reasons.
   nsIFrame* frame = DescendIntoBlockLevelFrame(aRS.frame);
   nsPresContext* prescontext = frame->PresContext();
   nsBlockFrame* block = nullptr;
-  if (0 == aRS.ComputedPhysicalBorderPadding().top) {
+  if (0 == aRS.ComputedLogicalBorderPadding().BStart(wm)) {
     block = nsLayoutUtils::GetAsBlock(frame);
     if (block) {
-      bool topMarginRoot, unused;
-      block->IsMarginRoot(&topMarginRoot, &unused);
-      if (topMarginRoot) {
+      bool bStartMarginRoot, unused;
+      block->IsMarginRoot(&bStartMarginRoot, &unused);
+      if (bStartMarginRoot) {
         block = nullptr;
       }
     }
@@ -112,7 +126,7 @@ nsBlockReflowContext::ComputeCollapsedTopMargin(const nsHTMLReflowState& aRS,
           line->MarkDirty();
           dirtiedLine = true;
         }
-        
+
         bool isEmpty;
         if (line->IsInline()) {
           isEmpty = line->IsEmpty();
@@ -129,7 +143,7 @@ nsBlockReflowContext::ComputeCollapsedTopMargin(const nsHTMLReflowState& aRS,
           // child blocks margin and so in so that we can look into
           // it. For its margins to be computed we need to have a reflow
           // state for it.
-          
+
           // We may have to construct an extra reflow state here if
           // we drilled down through a block wrapper. At the moment
           // we can only drill down one level so we only have to support
@@ -138,13 +152,13 @@ nsBlockReflowContext::ComputeCollapsedTopMargin(const nsHTMLReflowState& aRS,
           if (frame != aRS.frame) {
             NS_ASSERTION(frame->GetParent() == aRS.frame,
                          "Can only drill through one level of block wrapper");
-            nsSize availSpace(aRS.ComputedWidth(), aRS.ComputedHeight());
+            LogicalSize availSpace = aRS.ComputedSize(frame->GetWritingMode());
             outerReflowState = new nsHTMLReflowState(prescontext,
                                                      aRS, frame, availSpace);
           }
           {
-            nsSize availSpace(outerReflowState->ComputedWidth(),
-                              outerReflowState->ComputedHeight());
+            LogicalSize availSpace =
+              outerReflowState->ComputedSize(kid->GetWritingMode());
             nsHTMLReflowState innerReflowState(prescontext,
                                                *outerReflowState, kid,
                                                availSpace);
@@ -153,12 +167,18 @@ nsBlockReflowContext::ComputeCollapsedTopMargin(const nsHTMLReflowState& aRS,
             if (kid->StyleDisplay()->mBreakType != NS_STYLE_CLEAR_NONE) {
               *aMayNeedRetry = true;
             }
-            if (ComputeCollapsedTopMargin(innerReflowState, aMargin, aClearanceFrame, aMayNeedRetry, &isEmpty)) {
+            if (ComputeCollapsedBStartMargin(innerReflowState, aMargin,
+                                             aClearanceFrame, aMayNeedRetry,
+                                             &isEmpty)) {
               line->MarkDirty();
               dirtiedLine = true;
             }
-            if (isEmpty)
-              aMargin->Include(innerReflowState.ComputedPhysicalMargin().bottom);
+            if (isEmpty) {
+              WritingMode innerWM = innerReflowState.GetWritingMode();
+              LogicalMargin innerMargin =
+                innerReflowState.ComputedLogicalMargin().ConvertTo(parentWM, innerWM);
+              aMargin->Include(innerMargin.BEnd(parentWM));
+            }
           }
           if (outerReflowState != &aRS) {
             delete const_cast<nsHTMLReflowState*>(outerReflowState);
@@ -187,7 +207,7 @@ nsBlockReflowContext::ComputeCollapsedTopMargin(const nsHTMLReflowState& aRS,
     *aBlockIsEmpty = aRS.frame->IsEmpty();
   }
   
-#ifdef NOISY_VERTICAL_MARGINS
+#ifdef NOISY_BLOCKDIR_MARGINS
   nsFrame::ListTag(stdout, aRS.frame);
   printf(": => %d\n", aMargin->get());
 #endif
@@ -195,84 +215,100 @@ nsBlockReflowContext::ComputeCollapsedTopMargin(const nsHTMLReflowState& aRS,
   return dirtiedLine;
 }
 
-nsresult
-nsBlockReflowContext::ReflowBlock(const nsRect&       aSpace,
-                                  bool                aApplyTopMargin,
+void
+nsBlockReflowContext::ReflowBlock(const LogicalRect&  aSpace,
+                                  bool                aApplyBStartMargin,
                                   nsCollapsingMargin& aPrevMargin,
                                   nscoord             aClearance,
-                                  bool                aIsAdjacentWithTop,
+                                  bool                aIsAdjacentWithBStart,
                                   nsLineBox*          aLine,
                                   nsHTMLReflowState&  aFrameRS,
                                   nsReflowStatus&     aFrameReflowStatus,
                                   nsBlockReflowState& aState)
 {
-  nsresult rv = NS_OK;
   mFrame = aFrameRS.frame;
+  mWritingMode = aState.mReflowState.GetWritingMode();
+  mContainerWidth = aState.ContainerWidth();
   mSpace = aSpace;
 
-  if (!aIsAdjacentWithTop) {
+  if (!aIsAdjacentWithBStart) {
     aFrameRS.mFlags.mIsTopOfPage = false;  // make sure this is cleared
   }
 
-  if (aApplyTopMargin) {
-    mTopMargin = aPrevMargin;
+  if (aApplyBStartMargin) {
+    mBStartMargin = aPrevMargin;
 
-#ifdef NOISY_VERTICAL_MARGINS
+#ifdef NOISY_BLOCKDIR_MARGINS
     nsFrame::ListTag(stdout, mOuterReflowState.frame);
     printf(": reflowing ");
     nsFrame::ListTag(stdout, mFrame);
-    printf(" margin => %d, clearance => %d\n", mTopMargin.get(), aClearance);
+    printf(" margin => %d, clearance => %d\n", mBStartMargin.get(), aClearance);
 #endif
 
-    // Adjust the available height if its constrained so that the
+    // Adjust the available block size if it's constrained so that the
     // child frame doesn't think it can reflow into its margin area.
-    if (NS_UNCONSTRAINEDSIZE != aFrameRS.AvailableHeight()) {
-      aFrameRS.AvailableHeight() -= mTopMargin.get() + aClearance;
+    if (NS_UNCONSTRAINEDSIZE != aFrameRS.AvailableBSize()) {
+      aFrameRS.AvailableBSize() -= mBStartMargin.get() + aClearance;
     }
   }
 
-  nscoord tx = 0, ty = 0;
-  // The values of x and y do not matter for floats, so don't bother calculating
-  // them. Floats are guaranteed to have their own float manager, so tx and ty
-  // don't matter.  mX and mY don't matter becacuse they are only used in
-  // PlaceBlock, which is not used for floats.
+  nscoord tI = 0, tB = 0;
+  // The values of x and y do not matter for floats, so don't bother
+  // calculating them. Floats are guaranteed to have their own float
+  // manager, so tI and tB don't matter.  mICoord and mBCoord don't
+  // matter becacuse they are only used in PlaceBlock, which is not used
+  // for floats.
   if (aLine) {
-    // Compute x/y coordinate where reflow will begin. Use the rules
-    // from 10.3.3 to determine what to apply. At this point in the
-    // reflow auto left/right margins will have a zero value.
+    // Compute inline/block coordinate where reflow will begin. Use the
+    // rules from 10.3.3 to determine what to apply. At this point in the
+    // reflow auto inline-start/end margins will have a zero value.
 
-    mX = tx = mSpace.x + aFrameRS.ComputedPhysicalMargin().left;
-    mY = ty = mSpace.y + mTopMargin.get() + aClearance;
+    WritingMode frameWM = aFrameRS.GetWritingMode();
+    LogicalMargin usedMargin =
+      aFrameRS.ComputedLogicalMargin().ConvertTo(mWritingMode, frameWM);
+    mICoord = mSpace.IStart(mWritingMode) + usedMargin.IStart(mWritingMode);
+    mBCoord = mSpace.BStart(mWritingMode) + mBStartMargin.get() + aClearance;
+
+    LogicalRect space(mWritingMode, mICoord, mBCoord,
+                      mSpace.ISize(mWritingMode) -
+                      usedMargin.IStartEnd(mWritingMode),
+                      mSpace.BSize(mWritingMode) -
+                      usedMargin.BStartEnd(mWritingMode));
+    tI = space.LineLeft(mWritingMode, mContainerWidth);
+    tB = mBCoord;
 
     if ((mFrame->GetStateBits() & NS_BLOCK_FLOAT_MGR) == 0)
       aFrameRS.mBlockDelta =
-        mOuterReflowState.mBlockDelta + ty - aLine->BStart();
+        mOuterReflowState.mBlockDelta + mBCoord - aLine->BStart();
   }
 
   // Let frame know that we are reflowing it
   mFrame->WillReflow(mPresContext);
 
 #ifdef DEBUG
-  mMetrics.Width() = nscoord(0xdeadbeef);
-  mMetrics.Height() = nscoord(0xdeadbeef);
+  mMetrics.ISize(mWritingMode) = nscoord(0xdeadbeef);
+  mMetrics.BSize(mWritingMode) = nscoord(0xdeadbeef);
 #endif
 
-  mOuterReflowState.mFloatManager->Translate(tx, ty);
-  rv = mFrame->Reflow(mPresContext, mMetrics, aFrameRS, aFrameReflowStatus);
-  mOuterReflowState.mFloatManager->Translate(-tx, -ty);
+  mOuterReflowState.mFloatManager->Translate(tI, tB);
+  mFrame->Reflow(mPresContext, mMetrics, aFrameRS, aFrameReflowStatus);
+  mOuterReflowState.mFloatManager->Translate(-tI, -tB);
 
 #ifdef DEBUG
   if (!NS_INLINE_IS_BREAK_BEFORE(aFrameReflowStatus)) {
-    if (CRAZY_SIZE(mMetrics.Width()) || CRAZY_SIZE(mMetrics.Height())) {
+    if (CRAZY_SIZE(mMetrics.ISize(mWritingMode)) ||
+        CRAZY_SIZE(mMetrics.BSize(mWritingMode))) {
       printf("nsBlockReflowContext: ");
       nsFrame::ListTag(stdout, mFrame);
-      printf(" metrics=%d,%d!\n", mMetrics.Width(), mMetrics.Height());
+      printf(" metrics=%d,%d!\n",
+             mMetrics.ISize(mWritingMode), mMetrics.BSize(mWritingMode));
     }
-    if ((mMetrics.Width() == nscoord(0xdeadbeef)) ||
-        (mMetrics.Height() == nscoord(0xdeadbeef))) {
+    if ((mMetrics.ISize(mWritingMode) == nscoord(0xdeadbeef)) ||
+        (mMetrics.BSize(mWritingMode) == nscoord(0xdeadbeef))) {
       printf("nsBlockReflowContext: ");
       nsFrame::ListTag(stdout, mFrame);
-      printf(" didn't set w/h %d,%d!\n", mMetrics.Width(), mMetrics.Height());
+      printf(" didn't set i/b %d,%d!\n",
+             mMetrics.ISize(mWritingMode), mMetrics.BSize(mWritingMode));
     }
   }
 #endif
@@ -296,111 +332,119 @@ nsBlockReflowContext::ReflowBlock(const nsRect&       aSpace,
         // Floats will eventually be removed via nsBlockFrame::RemoveFloat
         // which detaches the placeholder from the float.
         nsOverflowContinuationTracker::AutoFinish fini(aState.mOverflowTracker, mFrame);
-        static_cast<nsContainerFrame*>(kidNextInFlow->GetParent())
-          ->DeleteNextInFlowChild(kidNextInFlow, true);
+        kidNextInFlow->GetParent()->DeleteNextInFlowChild(kidNextInFlow, true);
       }
     }
   }
-
-  return rv;
 }
 
 /**
  * Attempt to place the block frame within the available space.  If
- * it fits, apply horizontal positioning (CSS 10.3.3), collapse
- * margins (CSS2 8.3.1). Also apply relative positioning.
+ * it fits, apply inline-dir ("horizontal") positioning (CSS 10.3.3),
+ * collapse margins (CSS2 8.3.1). Also apply relative positioning.
  */
 bool
 nsBlockReflowContext::PlaceBlock(const nsHTMLReflowState&  aReflowState,
                                  bool                      aForceFit,
                                  nsLineBox*                aLine,
-                                 nsCollapsingMargin&       aBottomMarginResult,
+                                 nsCollapsingMargin&       aBEndMarginResult,
                                  nsOverflowAreas&          aOverflowAreas,
-                                 nsReflowStatus            aReflowStatus,
-                                 nscoord                   aContainerWidth)
+                                 nsReflowStatus            aReflowStatus)
 {
-  // Compute collapsed bottom margin value.
+  // Compute collapsed block-end margin value.
+  WritingMode wm = aReflowState.GetWritingMode();
+  WritingMode parentWM = mMetrics.GetWritingMode();
   if (NS_FRAME_IS_COMPLETE(aReflowStatus)) {
-    aBottomMarginResult = mMetrics.mCarriedOutBottomMargin;
-    aBottomMarginResult.Include(aReflowState.ComputedPhysicalMargin().bottom);
+    aBEndMarginResult = mMetrics.mCarriedOutBEndMargin;
+    aBEndMarginResult.Include(aReflowState.ComputedLogicalMargin().
+      ConvertTo(parentWM, wm).BEnd(parentWM));
   } else {
-    // The used bottom-margin is set to zero above a break.
-    aBottomMarginResult.Zero();
+    // The used block-end-margin is set to zero before a break.
+    aBEndMarginResult.Zero();
   }
 
-  nsPoint position(mX, mY);
   nscoord backupContainingBlockAdvance = 0;
 
-  // Check whether the block's bottom margin collapses with its top
+  // Check whether the block's block-end margin collapses with its block-start
   // margin. See CSS 2.1 section 8.3.1; those rules seem to match
-  // nsBlockFrame::IsEmpty(). Any such block must have zero height so
+  // nsBlockFrame::IsEmpty(). Any such block must have zero block-size so
   // check that first. Note that a block can have clearance and still
-  // have adjoining top/bottom margins, because the clearance goes
-  // above the top margin.
+  // have adjoining block-start/end margins, because the clearance goes
+  // above the block-start margin.
   // Mark the frame as non-dirty; it has been reflowed (or we wouldn't
   // be here), and we don't want to assert in CachedIsEmpty()
   mFrame->RemoveStateBits(NS_FRAME_IS_DIRTY);
-  bool empty = 0 == mMetrics.Height() && aLine->CachedIsEmpty();
+  bool empty = 0 == mMetrics.BSize(parentWM) && aLine->CachedIsEmpty();
   if (empty) {
-    // Collapse the bottom margin with the top margin that was already
-    // applied.
-    aBottomMarginResult.Include(mTopMargin);
+    // Collapse the block-end margin with the block-start margin that was
+    // already applied.
+    aBEndMarginResult.Include(mBStartMargin);
 
-#ifdef NOISY_VERTICAL_MARGINS
+#ifdef NOISY_BLOCKDIR_MARGINS
     printf("  ");
     nsFrame::ListTag(stdout, mOuterReflowState.frame);
     printf(": ");
     nsFrame::ListTag(stdout, mFrame);
-    printf(" -- collapsing top & bottom margin together; y=%d spaceY=%d\n",
-           position.y, mSpace.y);
+    printf(" -- collapsing block start & end margin together; BStart=%d spaceBStart=%d\n",
+           mBCoord, mSpace.BStart(mWritingMode));
 #endif
     // Section 8.3.1 of CSS 2.1 says that blocks with adjoining
-    // top/bottom margins whose top margin collapses with their
-    // parent's top margin should have their top border-edge at the
+    // "top/bottom" (i.e. block-start/end) margins whose top margin collapses
+    // with their parent's top margin should have their top border-edge at the
     // top border-edge of their parent. We actually don't have to do
     // anything special to make this happen. In that situation,
-    // nsBlockFrame::ShouldApplyTopMargin will have returned false,
-    // and mTopMargin and aClearance will have been zero in
+    // nsBlockFrame::ShouldApplyBStartMargin will have returned false,
+    // and mBStartMargin and aClearance will have been zero in
     // ReflowBlock.
 
-    // If we did apply our top margin, but now we're collapsing it
-    // into the bottom margin, we need to back up the containing
-    // block's y-advance by our top margin so that it doesn't get
+    // If we did apply our block-start margin, but now we're collapsing it
+    // into the block-end margin, we need to back up the containing
+    // block's bCoord-advance by our block-start margin so that it doesn't get
     // counted twice. Note that here we're allowing the line's bounds
     // to become different from the block's position; we do this
     // because the containing block will place the next line at the
-    // line's YMost, and it must place the next line at a different
+    // line's BEnd, and it must place the next line at a different
     // point from where this empty block will be.
-    backupContainingBlockAdvance = mTopMargin.get();
+    backupContainingBlockAdvance = mBStartMargin.get();
   }
 
   // See if the frame fit. If it's the first frame or empty then it
-  // always fits. If the height is unconstrained then it always fits,
-  // even if there's some sort of integer overflow that makes y +
-  // mMetrics.Height() appear to go beyond the available height.
-  if (!empty && !aForceFit && mSpace.height != NS_UNCONSTRAINEDSIZE) {
-    nscoord yMost = position.y - backupContainingBlockAdvance + mMetrics.Height();
-    if (yMost > mSpace.YMost()) {
+  // always fits. If the block-size is unconstrained then it always fits,
+  // even if there's some sort of integer overflow that makes bCoord +
+  // mMetrics.BSize() appear to go beyond the available block size.
+  if (!empty && !aForceFit &&
+      mSpace.BSize(mWritingMode) != NS_UNCONSTRAINEDSIZE) {
+    nscoord bEnd = mBCoord -
+                   backupContainingBlockAdvance + mMetrics.BSize(mWritingMode);
+    if (bEnd > mSpace.BEnd(mWritingMode)) {
       // didn't fit, we must acquit.
-      mFrame->DidReflow(mPresContext, &aReflowState, nsDidReflowStatus::FINISHED);
+      mFrame->DidReflow(mPresContext, &aReflowState,
+                        nsDidReflowStatus::FINISHED);
       return false;
     }
   }
 
-  aLine->SetBounds(aReflowState.GetWritingMode(),
-                   nsRect(position.x,
-                          position.y - backupContainingBlockAdvance,
-                          mMetrics.Width(),
-                          mMetrics.Height()),
-                   aContainerWidth);
+  aLine->SetBounds(mWritingMode,
+                   mICoord, mBCoord - backupContainingBlockAdvance,
+                   mMetrics.ISize(mWritingMode), mMetrics.BSize(mWritingMode),
+                   mContainerWidth);
 
-  aReflowState.ApplyRelativePositioning(&position);
+  WritingMode frameWM = mFrame->GetWritingMode();
+  LogicalPoint logPos =
+    LogicalPoint(mWritingMode, mICoord, mBCoord).
+      ConvertTo(frameWM, mWritingMode, mContainerWidth - mMetrics.Width());
+
+  // ApplyRelativePositioning in right-to-left writing modes needs to
+  // know the updated frame width
+  mFrame->SetSize(mWritingMode, mMetrics.Size(mWritingMode));
+  aReflowState.ApplyRelativePositioning(&logPos, mContainerWidth);
 
   // Now place the frame and complete the reflow process
   nsContainerFrame::FinishReflowChild(mFrame, mPresContext, mMetrics,
-                                      &aReflowState, position.x, position.y, 0);
+                                      &aReflowState, frameWM, logPos,
+                                      mContainerWidth, 0);
 
-  aOverflowAreas = mMetrics.mOverflowAreas + position;
+  aOverflowAreas = mMetrics.mOverflowAreas + mFrame->GetPosition();
 
   return true;
 }
