@@ -20,7 +20,6 @@
 #include "webrtc/voice_engine/include/voe_audio_processing.h"
 #include "webrtc/voice_engine/include/voe_base.h"
 #include "webrtc/voice_engine/include/voe_codec.h"
-#include "webrtc/voice_engine/include/voe_encryption.h"
 #include "webrtc/voice_engine/include/voe_file.h"
 #include "webrtc/voice_engine/include/voe_hardware.h"
 #include "webrtc/voice_engine/include/voe_network.h"
@@ -89,13 +88,6 @@
                             "RTP / RTCP pointer doesn't exist"); \
         return -1; \
     }
-#define VALIDATE_ENCRYPT_POINTER \
-    if (!veData1.encrypt) \
-    { \
-        __android_log_write(ANDROID_LOG_ERROR, WEBRTC_LOG_TAG, \
-                            "Encrypt pointer doesn't exist"); \
-        return -1; \
-    }
 
 // Register functions in JNI_OnLoad()
 // How do we ensure that VoE is deleted? JNI_OnUnload?
@@ -116,32 +108,6 @@ enum TestType
   CPU      = 4
 };
 
-// ExtendedSelection enumerator
-enum ExtendedSelection
-{
-   XSEL_Invalid = -1,
-   XSEL_None = 0,
-   XSEL_All,
-   XSEL_Base,
-   XSEL_CallReport,
-   XSEL_Codec,
-   XSEL_DTMF,
-   XSEL_Encryption,
-   XSEL_ExternalMedia,
-   XSEL_File,
-   XSEL_Hardware,
-   XSEL_NetEqStats,
-   XSEL_Network,
-   XSEL_PTT,
-   XSEL_RTP_RTCP,
-   XSEL_VideoSync,
-   XSEL_VideoSyncExtended,
-   XSEL_VolumeControl,
-   XSEL_VQE,
-   XSEL_APM,
-   XSEL_VQMon
-};
-
 using namespace webrtc;
 
 class my_transportation;
@@ -160,7 +126,6 @@ typedef struct
     VoEVolumeControl* volume;
     VoEHardware* hardware;
     VoERTP_RTCP* rtp_rtcp;
-    VoEEncryption* encrypt;
     // Other
     my_transportation* extTrans;
     JavaVM* jvm;
@@ -174,8 +139,8 @@ class my_transportation : public Transport
       netw(network) {
   }
 
-  int SendPacket(int channel,const void *data,int len);
-  int SendRTCPPacket(int channel, const void *data, int len);
+  virtual int SendPacket(int channel,const void *data,int len) OVERRIDE;
+  virtual int SendRTCPPacket(int channel, const void *data, int len) OVERRIDE;
  private:
   VoENetwork * netw;
 };
@@ -447,80 +412,12 @@ JNIEXPORT jint JNICALL Java_org_webrtc_voiceengine_test_AndroidTest_RunAutoTest(
             return -1;
     }
 
-    ExtendedSelection xsel(XSEL_Invalid);
-
-    switch (extendedSel)
-    {
-        case 0:
-            xsel = XSEL_None;
-            break;
-        case 1:
-            xsel = XSEL_All;
-            break;
-        case 2:
-            xsel = XSEL_Base;
-            break;
-        case 3:
-            xsel = XSEL_CallReport;
-            break;
-        case 4:
-            xsel = XSEL_Codec;
-            break;
-        case 5:
-            xsel = XSEL_DTMF;
-            break;
-        case 6:
-            xsel = XSEL_Encryption;
-            break;
-        case 7:
-            xsel = XSEL_ExternalMedia;
-            break;
-        case 8:
-            xsel = XSEL_File;
-            break;
-        case 9:
-            xsel = XSEL_Hardware;
-            break;
-        case 10:
-            xsel = XSEL_NetEqStats;
-            break;
-        case 11:
-            xsel = XSEL_Network;
-            break;
-        case 12:
-            xsel = XSEL_PTT;
-            break;
-        case 13:
-            xsel = XSEL_RTP_RTCP;
-            break;
-        case 14:
-            xsel = XSEL_VideoSync;
-            break;
-        case 15:
-            xsel = XSEL_VideoSyncExtended;
-            break;
-        case 16:
-            xsel = XSEL_VolumeControl;
-            break;
-        case 17:
-            xsel = XSEL_APM;
-            break;
-        case 18:
-            xsel = XSEL_VQMon;
-            break;
-        default:
-            xsel = XSEL_Invalid;
-            __android_log_write(ANDROID_LOG_ERROR, WEBRTC_LOG_TAG,
-                                "RunAutoTest - Invalid extendedType");
-            return -1;
-    }
-
     // Set instance independent Java objects
     VoiceEngine::SetAndroidObjects(veData1.jvm, env, context);
 
     // Call voe test interface function
     // TODO(leozwang) add autotest setAndroidObjects(veData1.jvm, context);
-    // jint retVal = runAutoTest(tType, xsel);
+    // jint retVal = runAutoTest(tType);
 
     // Clear instance independent Java objects
     VoiceEngine::SetAndroidObjects(NULL, NULL, NULL);
@@ -813,10 +710,10 @@ JNIEXPORT jint JNICALL Java_org_webrtc_voiceengine_test_AndroidTest_StartSend(
      return -1;
      } */
 
-    /* if (veData1.rtp_rtcp->SetFECStatus(channel, 1) != 0)
+    /* if (veData1.rtp_rtcp->SetREDStatus(channel, 1) != 0)
      {
      __android_log_write(ANDROID_LOG_ERROR, WEBRTC_LOG_TAG,
-         "Failed to enable FEC");
+         "Failed to enable RED");
      return -1;
      } */
 
@@ -858,10 +755,10 @@ JNIEXPORT jint JNICALL Java_org_webrtc_voiceengine_test_AndroidTest_StopSend(
         jobject,
         jint channel)
 {
-    /* if (veData1.rtp_rtcp->SetFECStatus(channel, 0) != 0)
+    /* if (veData1.rtp_rtcp->SetREDStatus(channel, 0) != 0)
      {
      __android_log_write(ANDROID_LOG_ERROR, WEBRTC_LOG_TAG,
-         "Failed to disable FEC");
+         "Failed to disable RED");
      return -1;
      } */
 
@@ -1245,21 +1142,21 @@ Java_org_webrtc_voiceengine_test_AndroidTest_SetLoudspeakerStatus(
 
     /*VALIDATE_RTP_RTCP_POINTER;
 
-     if (veData1.rtp_rtcp->SetFECStatus(0, enable, -1) != 0)
+     if (veData1.rtp_rtcp->SetREDStatus(0, enable, -1) != 0)
      {
      __android_log_write(ANDROID_LOG_ERROR, WEBRTC_LOG_TAG,
-         "Could not set FEC");
+         "Could not set RED");
      return -1;
      }
      else if(enable)
      {
      __android_log_write(ANDROID_LOG_ERROR, WEBRTC_LOG_TAG,
-         "Could enable FEC");
+         "Could enable RED");
      }
      else
      {
      __android_log_write(ANDROID_LOG_ERROR, WEBRTC_LOG_TAG,
-         "Could disable FEC");
+         "Could disable RED");
      }*/
 
     return 0;
@@ -1348,15 +1245,6 @@ bool GetSubApis(VoiceEngineData &veData)
         getOK = false;
     }
 
-    // Encrypt
-    veData.encrypt = VoEEncryption::GetInterface(veData.ve);
-    if (!veData.encrypt)
-    {
-        __android_log_write(ANDROID_LOG_ERROR, WEBRTC_LOG_TAG,
-                            "Get encrypt sub-API failed");
-        getOK = false;
-    }
-
     return getOK;
 }
 
@@ -1487,20 +1375,5 @@ bool ReleaseSubApis(VoiceEngineData &veData)
         }
     }
 
-    // Encrypt
-    if (veData.encrypt)
-    {
-        if (0 != veData.encrypt->Release())
-        {
-            __android_log_write(ANDROID_LOG_ERROR, WEBRTC_LOG_TAG,
-                                "Release encrypt sub-API failed");
-            releaseOK = false;
-        }
-        else
-        {
-            veData.encrypt = NULL;
-        }
-    }
-
     return releaseOK;
 }
