@@ -14,6 +14,7 @@ import java.util.Map.Entry;
 import org.json.JSONArray;
 import org.json.JSONException;
 import org.json.JSONObject;
+import org.mozilla.gecko.AppConstants.Versions;
 import org.mozilla.gecko.util.GeckoEventListener;
 import org.mozilla.gecko.util.ThreadUtils;
 
@@ -29,7 +30,6 @@ import android.content.DialogInterface;
 import android.content.OperationApplicationException;
 import android.database.Cursor;
 import android.net.Uri;
-import android.os.Build;
 import android.os.RemoteException;
 import android.provider.ContactsContract;
 import android.provider.ContactsContract.CommonDataKinds.BaseTypes;
@@ -90,36 +90,37 @@ public class ContactService implements GeckoEventListener {
     private HashMap<String, Integer> mWebsiteTypesMap;
     private HashMap<String, Integer> mImTypesMap;
 
-    private ContentResolver mContentResolver;
-    private GeckoApp mActivity;
+    private final ContentResolver mContentResolver;
+    private final GeckoApp mActivity;
 
     ContactService(EventDispatcher eventDispatcher, GeckoApp activity) {
         mEventDispatcher = eventDispatcher;
         mActivity = activity;
         mContentResolver = mActivity.getContentResolver();
-        mGotDeviceAccount = false;
 
-        registerEventListener("Android:Contacts:Clear");
-        registerEventListener("Android:Contacts:Find");
-        registerEventListener("Android:Contacts:GetAll");
-        registerEventListener("Android:Contacts:GetCount");
-        registerEventListener("Android:Contact:Remove");
-        registerEventListener("Android:Contact:Save");
+        EventDispatcher.getInstance().registerGeckoThreadListener(this,
+            "Android:Contacts:Clear",
+            "Android:Contacts:Find",
+            "Android:Contacts:GetAll",
+            "Android:Contacts:GetCount",
+            "Android:Contact:Remove",
+            "Android:Contact:Save");
     }
 
     public void destroy() {
-        unregisterEventListener("Android:Contacts:Clear");
-        unregisterEventListener("Android:Contacts:Find");
-        unregisterEventListener("Android:Contacts:GetAll");
-        unregisterEventListener("Android:Contacts:GetCount");
-        unregisterEventListener("Android:Contact:Remove");
-        unregisterEventListener("Android:Contact:Save");
+        EventDispatcher.getInstance().unregisterGeckoThreadListener(this,
+            "Android:Contacts:Clear",
+            "Android:Contacts:Find",
+            "Android:Contacts:GetAll",
+            "Android:Contacts:GetCount",
+            "Android:Contact:Remove",
+            "Android:Contact:Save");
     }
 
     @Override
     public void handleMessage(final String event, final JSONObject message) {
         // If the account chooser dialog needs shown to the user, the message handling becomes
-        // asychronous so it needs posted to a background thread from the UI thread when the
+        // asynchronous so it needs posted to a background thread from the UI thread when the
         // account chooser dialog is dismissed by the user.
         Runnable handleMessage = new Runnable() {
             @Override
@@ -246,9 +247,7 @@ public class ContactService implements GeckoEventListener {
                 // Truncate the raw contacts IDs array if necessary
                 if (filterLimit > 0 && allRawContactIds.length > filterLimit) {
                     long[] truncatedRawContactIds = new long[filterLimit];
-                    for (int i = 0; i < filterLimit; i++) {
-                        truncatedRawContactIds[i] = allRawContactIds[i];
-                    }
+                    System.arraycopy(allRawContactIds, 0, truncatedRawContactIds, 0, filterLimit);
                     return truncatedRawContactIds;
                 }
                 return allRawContactIds;
@@ -401,41 +400,47 @@ public class ContactService implements GeckoEventListener {
         filterValue = filterValue.toLowerCase();
         databaseValue = databaseValue.toLowerCase();
 
-        if ("match".equals(filterOp)) {
-            // If substring matching is a positive number, only pay attention to the last X characters
-            // of both the filter and database values
-            if (substringMatching > 0) {
-                databaseValue = substringStartFromEnd(cleanPhoneNumber(databaseValue), substringMatching);
-                filterValue = substringStartFromEnd(cleanPhoneNumber(filterValue), substringMatching);
-                return databaseValue.startsWith(filterValue);
-            }
-            return databaseValue.equals(filterValue);
-        } else if ("equals".equals(filterOp)) {
-            if (isPhone) {
-                return PhoneNumberUtils.compare(filterValue, databaseValue);
-            }
-            return databaseValue.equals(filterValue);
-        } else if ("contains".equals(filterOp)) {
-            if (isPhone) {
-                filterValue = cleanPhoneNumber(filterValue);
-                databaseValue = cleanPhoneNumber(databaseValue);
-            }
-            return databaseValue.contains(filterValue);
-        } else if ("startsWith".equals(filterOp)) {
-            // If a phone number, remove non-dialable characters and then only pay attention to
-            // the last X digits given by the substring matching values (see bug 877302)
-            if (isPhone) {
-                String cleanedDatabasePhone = cleanPhoneNumber(databaseValue);
+        switch (filterOp) {
+            case "match":
+                // If substring matching is a positive number, only pay attention to the last X characters
+                // of both the filter and database values
                 if (substringMatching > 0) {
-                    cleanedDatabasePhone = substringStartFromEnd(cleanedDatabasePhone, substringMatching);
+                    databaseValue = substringStartFromEnd(cleanPhoneNumber(databaseValue), substringMatching);
+                    filterValue = substringStartFromEnd(cleanPhoneNumber(filterValue), substringMatching);
+                    return databaseValue.startsWith(filterValue);
                 }
 
-                if (cleanedDatabasePhone.startsWith(filterValue)) {
-                    return true;
+                return databaseValue.equals(filterValue);
+            case "equals":
+                if (isPhone) {
+                    return PhoneNumberUtils.compare(filterValue, databaseValue);
                 }
-            }
-            return databaseValue.startsWith(filterValue);
+
+                return databaseValue.equals(filterValue);
+            case "contains":
+                if (isPhone) {
+                    filterValue = cleanPhoneNumber(filterValue);
+                    databaseValue = cleanPhoneNumber(databaseValue);
+                }
+
+                return databaseValue.contains(filterValue);
+            case "startsWith":
+                // If a phone number, remove non-dialable characters and then only pay attention to
+                // the last X digits given by the substring matching values (see bug 877302)
+                if (isPhone) {
+                    String cleanedDatabasePhone = cleanPhoneNumber(databaseValue);
+                    if (substringMatching > 0) {
+                        cleanedDatabasePhone = substringStartFromEnd(cleanedDatabasePhone, substringMatching);
+                    }
+
+                    if (cleanedDatabasePhone.startsWith(filterValue)) {
+                        return true;
+                    }
+                }
+
+                return databaseValue.startsWith(filterValue);
         }
+
         return false;
     }
 
@@ -658,7 +663,7 @@ public class ContactService implements GeckoEventListener {
     }
 
     private boolean bool(int integer) {
-        return integer != 0 ? true : false;
+        return integer != 0;
     }
 
     private void getGenericDataAsJSONObject(Cursor cursor, JSONArray array, final String dataColumn,
@@ -670,7 +675,7 @@ public class ContactService implements GeckoEventListener {
         if (typeConstant == BaseTypes.TYPE_CUSTOM) {
             type = cursor.getString(cursor.getColumnIndex(typeLabelColumn));
         } else {
-            type = getKeyFromMapValue(typeMap, Integer.valueOf(typeConstant));
+            type = getKeyFromMapValue(typeMap, typeConstant);
         }
 
         // Since an object may have multiple types, it may have already been added,
@@ -711,7 +716,7 @@ public class ContactService implements GeckoEventListener {
         if (typeConstant == Phone.TYPE_CUSTOM) {
             type = cursor.getString(cursor.getColumnIndex(Phone.LABEL));
         } else {
-            type = getKeyFromMapValue(mPhoneTypesMap, Integer.valueOf(typeConstant));
+            type = getKeyFromMapValue(mPhoneTypesMap, typeConstant);
         }
 
         // Since a phone may have multiple types, it may have already been added,
@@ -758,7 +763,7 @@ public class ContactService implements GeckoEventListener {
         if (typeConstant == StructuredPostal.TYPE_CUSTOM) {
             type = cursor.getString(cursor.getColumnIndex(StructuredPostal.LABEL));
         } else {
-            type = getKeyFromMapValue(mAddressTypesMap, Integer.valueOf(typeConstant));
+            type = getKeyFromMapValue(mAddressTypesMap, typeConstant);
         }
 
         // Since an email may have multiple types, it may have already been added,
@@ -983,7 +988,7 @@ public class ContactService implements GeckoEventListener {
         }
 
         String returnStatus = "KO";
-        Long newRawContactId = new Long(-1);
+        Long newRawContactId = -1L;
 
         // Insert the contact!
         ContentProviderResult[] insertResults = applyBatch(newContactOptions);
@@ -1019,7 +1024,7 @@ public class ContactService implements GeckoEventListener {
         // row from the contacts data table that belongs to the contact, and insert the new
         // fields. But then why not just delete all the data from the data in one go and
         // insert the new data in another? Because if all the data relating to a contact is
-        // deleted, Android will "conviently" remove the ID making it impossible to insert data
+        // deleted, Android will "conveniently" remove the ID making it impossible to insert data
         // under the old ID. To work around this, we put a GNU contact flag in the database
 
         ContentProviderOperation removeOptions = ContentProviderOperation.newDelete(Data.CONTENT_URI)
@@ -1475,7 +1480,7 @@ public class ContactService implements GeckoEventListener {
 
     private void getContactsCount(final String requestID) {
         Cursor cursor = getAllRawContactIdsCursor();
-        Integer numContacts = Integer.valueOf(cursor.getCount());
+        Integer numContacts = cursor.getCount();
         cursor.close();
 
         sendCallbackToJavascript("Android:Contacts:Count", requestID, new String[] {"count"},
@@ -1507,14 +1512,6 @@ public class ContactService implements GeckoEventListener {
         }
     }
 
-    private void registerEventListener(final String event) {
-        mEventDispatcher.registerEventListener(event, this);
-    }
-
-    private void unregisterEventListener(final String event) {
-        mEventDispatcher.unregisterEventListener(event, this);
-    }
-
     private ContentProviderResult[] applyBatch(ArrayList<ContentProviderOperation> operations) {
         try {
             return mContentResolver.applyBatch(ContactsContract.AUTHORITY, operations);
@@ -1552,7 +1549,7 @@ public class ContactService implements GeckoEventListener {
         }
 
         final AlertDialog.Builder builder = new AlertDialog.Builder(mActivity);
-        builder.setTitle(mActivity.getResources().getString(R.string.contacts_account_chooser_dialog_title))
+        builder.setTitle(mActivity.getResources().getString(R.string.contacts_account_chooser_dialog_title2))
             .setSingleChoiceItems(accountNames, 0, new DialogInterface.OnClickListener() {
                 @Override
                 public void onClick(DialogInterface dialog, int position) {
@@ -1565,6 +1562,7 @@ public class ContactService implements GeckoEventListener {
             });
 
         mActivity.runOnUiThread(new Runnable() {
+            @Override
             public void run() {
                 builder.show();
             }
@@ -1634,7 +1632,8 @@ public class ContactService implements GeckoEventListener {
     private static boolean isAutoAddGroup(Cursor cursor) {
         // For Honeycomb and up, the default group is the first one which has the AUTO_ADD flag set.
         // For everything below Honeycomb, use the default "System Group: My Contacts" group
-        return (Build.VERSION.SDK_INT >= 11 && !cursor.isNull(GROUP_AUTO_ADD) &&
+        return (Versions.feature11Plus &&
+                !cursor.isNull(GROUP_AUTO_ADD) &&
                 cursor.getInt(GROUP_AUTO_ADD) != 0);
     }
 
@@ -1687,7 +1686,7 @@ public class ContactService implements GeckoEventListener {
             Groups.ACCOUNT_TYPE,
             Groups._ID,
             Groups.TITLE,
-            (Build.VERSION.SDK_INT >= Build.VERSION_CODES.HONEYCOMB ? Groups.AUTO_ADD : Groups._ID)
+            (Versions.feature11Plus ? Groups.AUTO_ADD : Groups._ID)
         };
 
         if (selectArg != null) {
@@ -1829,7 +1828,7 @@ public class ContactService implements GeckoEventListener {
         }
     }
 
-    private static String getKeyFromMapValue(final HashMap<String, Integer> map, Integer value) {
+    private static String getKeyFromMapValue(final HashMap<String, Integer> map, int value) {
         for (Entry<String, Integer> entry : map.entrySet()) {
             if (value == entry.getValue()) {
                 return entry.getKey();
@@ -1904,7 +1903,7 @@ public class ContactService implements GeckoEventListener {
     private int getAddressType(String addressType) {
         initAddressTypesMap();
         Integer type = mAddressTypesMap.get(addressType.toLowerCase());
-        return (type != null ? Integer.valueOf(type) : StructuredPostal.TYPE_CUSTOM);
+        return type != null ? type : StructuredPostal.TYPE_CUSTOM;
     }
 
     private void initAddressTypesMap() {
@@ -1920,7 +1919,7 @@ public class ContactService implements GeckoEventListener {
     private int getPhoneType(String phoneType) {
         initPhoneTypesMap();
         Integer type = mPhoneTypesMap.get(phoneType.toLowerCase());
-        return (type != null ? Integer.valueOf(type) : Phone.TYPE_CUSTOM);
+        return type != null ? type : Phone.TYPE_CUSTOM;
     }
 
     private void initPhoneTypesMap() {
@@ -1955,7 +1954,7 @@ public class ContactService implements GeckoEventListener {
     private int getEmailType(String emailType) {
         initEmailTypesMap();
         Integer type = mEmailTypesMap.get(emailType.toLowerCase());
-        return (type != null ? Integer.valueOf(type) : Email.TYPE_CUSTOM);
+        return type != null ? type : Email.TYPE_CUSTOM;
     }
 
     private void initEmailTypesMap() {
@@ -1969,10 +1968,10 @@ public class ContactService implements GeckoEventListener {
         mEmailTypesMap.put("work", Email.TYPE_WORK);
     }
 
-    private int getWebsiteType(String webisteType) {
+    private int getWebsiteType(String websiteType) {
         initWebsiteTypesMap();
-        Integer type = mWebsiteTypesMap.get(webisteType.toLowerCase());
-        return (type != null ? Integer.valueOf(type) : Website.TYPE_CUSTOM);
+        Integer type = mWebsiteTypesMap.get(websiteType.toLowerCase());
+        return type != null ? type : Website.TYPE_CUSTOM;
     }
 
     private void initWebsiteTypesMap() {
@@ -1992,7 +1991,7 @@ public class ContactService implements GeckoEventListener {
     private int getImType(String imType) {
         initImTypesMap();
         Integer type = mImTypesMap.get(imType.toLowerCase());
-        return (type != null ? Integer.valueOf(type) : Im.TYPE_CUSTOM);
+        return type != null ? type : Im.TYPE_CUSTOM;
     }
 
     private void initImTypesMap() {
