@@ -3,6 +3,7 @@
 # You can obtain one at http://mozilla.org/MPL/2.0/.
 
 import os
+import re
 import select
 import signal
 import subprocess
@@ -11,7 +12,7 @@ import threading
 import time
 import traceback
 from Queue import Queue
-from datetime import datetime, timedelta
+from datetime import datetime
 __all__ = ['ProcessHandlerMixin', 'ProcessHandler']
 
 # Set the MOZPROCESS_DEBUG environment variable to 1 to see some debugging output
@@ -23,7 +24,7 @@ isPosix = os.name == "posix" # includes MacOS X
 
 if isWin:
     import ctypes, ctypes.wintypes, msvcrt
-    from ctypes import sizeof, addressof, c_ulong, byref, POINTER, WinError, c_longlong
+    from ctypes import sizeof, addressof, c_ulong, byref, WinError, c_longlong
     import winprocess
     from qijo import JobObjectAssociateCompletionPortInformation,\
     JOBOBJECT_ASSOCIATE_COMPLETION_PORT, JobObjectExtendedLimitInformation,\
@@ -94,7 +95,7 @@ class ProcessHandlerMixin(object):
                                           preexec_fn, close_fds,
                                           shell, cwd, env,
                                           universal_newlines, startupinfo, creationflags)
-            except OSError, e:
+            except OSError:
                 print >> sys.stderr, args
                 raise
 
@@ -204,11 +205,12 @@ class ProcessHandlerMixin(object):
                     comspec = os.environ.get("COMSPEC", "cmd.exe")
                     args = comspec + " /c " + args
 
-                # determine if we can create create a job
-                canCreateJob = winprocess.CanCreateJobObject()
+                # Determine if we can create a job or create nested jobs.
+                can_create_job = winprocess.CanCreateJobObject()
+                can_nest_jobs = self._can_nest_jobs()
 
                 # Ensure we write a warning message if we are falling back
-                if not canCreateJob and not self._ignore_children:
+                if not (can_create_job or can_nest_jobs) and not self._ignore_children:
                     # We can't create job objects AND the user wanted us to
                     # Warn the user about this.
                     print >> sys.stderr, "ProcessManager UNABLE to use job objects to manage child processes"
@@ -216,9 +218,9 @@ class ProcessHandlerMixin(object):
                 # set process creation flags
                 creationflags |= winprocess.CREATE_SUSPENDED
                 creationflags |= winprocess.CREATE_UNICODE_ENVIRONMENT
-                if canCreateJob:
+                if can_create_job:
                     creationflags |= winprocess.CREATE_BREAKAWAY_FROM_JOB
-                else:
+                if not (can_create_job or can_nest_jobs):
                     # Since we've warned, we just log info here to inform you
                     # of the consequence of setting ignore_children = True
                     print "ProcessManager NOT managing child processes"
@@ -237,7 +239,7 @@ class ProcessHandlerMixin(object):
                 self.pid = pid
                 self.tid = tid
 
-                if not self._ignore_children and canCreateJob:
+                if not self._ignore_children and (can_create_job or can_nest_jobs):
                     try:
                         # We create a new job for this process, so that we can kill
                         # the process and any sub-processes
@@ -311,6 +313,17 @@ falling back to not using job objects for managing child processes"""
                     if i is not None:
                         i.Close()
 
+            def _can_nest_jobs(self):
+                # Per:
+                # https://msdn.microsoft.com/en-us/library/windows/desktop/hh448388%28v=vs.85%29.aspx
+                # Nesting jobs came in with windows versions starting with 6.2 according to the table
+                # on this page:
+                # https://msdn.microsoft.com/en-us/library/ms724834%28v=vs.85%29.aspx
+                winver = sys.getwindowsversion()
+                return (winver.major > 6 or
+                        winver.major == 6 and winver.minor >= 2)
+
+
             # Windows Process Manager - watches the IO Completion Port and
             # keeps track of child processes
             def _procmgr(self):
@@ -362,6 +375,7 @@ falling back to not using job objects for managing child processes"""
                         if errcode == winprocess.ERROR_ABANDONED_WAIT_0:
                             # Then something has killed the port, break the loop
                             print >> sys.stderr, "IO Completion Port unexpectedly closed"
+                            self._process_events.put({self.pid: 'FINISHED'})
                             break
                         elif errcode == winprocess.WAIT_TIMEOUT:
                             # Timeouts are expected, just keep on polling
@@ -693,7 +707,7 @@ falling back to not using job objects for managing child processes"""
             return self.wait()
         except AttributeError:
             # Try to print a relevant error message.
-            if not self.proc:
+            if not hasattr(self, 'proc'):
                 print >> sys.stderr, "Unable to kill Process because call to ProcessHandler constructor failed."
             else:
                 raise
@@ -811,7 +825,7 @@ falling back to not using job objects for managing child processes"""
         - '0' if the process ended without failures
 
         """
-        if self.outThread:
+        if self.outThread and self.outThread is not threading.current_thread():
             # Thread.join() blocks the main thread until outThread is finished
             # wake up once a second in case a keyboard interrupt is sent
             count = 0
@@ -821,7 +835,8 @@ falling back to not using job objects for managing child processes"""
                 if timeout and count > timeout:
                     return None
 
-        return self.proc.wait()
+        self.returncode = self.proc.wait()
+        return self.returncode
 
     # TODO Remove this method when consumers have been fixed
     def waitForFinish(self, timeout=None):
@@ -837,10 +852,17 @@ falling back to not using job objects for managing child processes"""
         PeekNamedPipe = ctypes.windll.kernel32.PeekNamedPipe
         GetLastError = ctypes.windll.kernel32.GetLastError
 
+        @staticmethod
+        def _normalize_newline(line):
+            # adb on windows returns \r\r\n at the end of each line, to get around
+            # this normalize all newlines to have a unix-style '\n'
+            # http://src.chromium.org/viewvc/chrome/trunk/src/build/android/pylib/android_commands.py#l1944
+            return re.sub(r'\r+\n?$', '\n', line)
+
         def _readWithTimeout(self, f, timeout):
             if timeout is None:
                 # shortcut to allow callers to pass in "None" for no timeout.
-                return (f.readline(), False)
+                return (self._normalize_newline(f.readline()), False)
             x = msvcrt.get_osfhandle(f.fileno())
             l = ctypes.c_long()
             done = time.time() + timeout
@@ -854,7 +876,7 @@ falling back to not using job objects for managing child processes"""
                 if l.value > 0:
                     # we're assuming that the output is line-buffered,
                     # which is not unreasonable
-                    return (f.readline(), False)
+                    return (self._normalize_newline(f.readline()), False)
                 time.sleep(0.01)
             return ('', True)
 
@@ -913,7 +935,12 @@ class StreamOutput(object):
         self.stream = stream
 
     def __call__(self, line):
-        self.stream.write(line + '\n')
+        try:
+            self.stream.write(line + '\n')
+        except UnicodeDecodeError:
+            # TODO: Workaround for bug #991866 to make sure we can display when
+            # when normal UTF-8 display is failing
+            self.stream.write(line.decode('iso8859-1') + '\n')
         self.stream.flush()
 
 class LogOutput(StreamOutput):
@@ -946,7 +973,7 @@ class ProcessHandler(ProcessHandlerMixin):
     appended to the given file.
     """
 
-    def __init__(self, cmd, logfile=None, stream=None,  storeOutput=True, **kwargs):
+    def __init__(self, cmd, logfile=None, stream=None, storeOutput=True, **kwargs):
         kwargs.setdefault('processOutputLine', [])
         if callable(kwargs['processOutputLine']):
             kwargs['processOutputLine'] = [kwargs['processOutputLine']]
