@@ -12,8 +12,8 @@
 class nsMIMEInfoUnix : public nsMIMEInfoImpl
 {
 public:
-  nsMIMEInfoUnix(const char *aMIMEType = "") : nsMIMEInfoImpl(aMIMEType) {}
-  nsMIMEInfoUnix(const nsACString& aMIMEType) : nsMIMEInfoImpl(aMIMEType) {}
+  explicit nsMIMEInfoUnix(const char *aMIMEType = "") : nsMIMEInfoImpl(aMIMEType) {}
+  explicit nsMIMEInfoUnix(const nsACString& aMIMEType) : nsMIMEInfoImpl(aMIMEType) {}
   nsMIMEInfoUnix(const nsACString& aType, HandlerClass aClass) :
     nsMIMEInfoImpl(aType, aClass) {}
   static bool HandlerExists(const char *aProtocolScheme);
@@ -21,9 +21,9 @@ public:
 protected:
   NS_IMETHOD GetHasDefaultHandler(bool *_retval);
 
-  virtual NS_HIDDEN_(nsresult) LoadUriInternal(nsIURI *aURI);
+  virtual nsresult LoadUriInternal(nsIURI *aURI);
 
-  virtual NS_HIDDEN_(nsresult) LaunchDefaultWithFile(nsIFile *aFile);
+  virtual nsresult LaunchDefaultWithFile(nsIFile *aFile);
 #if defined(MOZ_ENABLE_CONTENTACTION)
   NS_IMETHOD GetPossibleApplicationHandlers(nsIMutableArray * *aPossibleAppHandlers);
 #endif
